@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/bin/sh
 # Run this to generate all the initial makefiles, etc.
 
 DIE=0
@@ -17,7 +17,7 @@ fi
 mkdir -p gst-libs/ext/ffmpeg/ffmpeg
 
 # source helper functions
-if test ! -e common/gst-autogen.sh;
+if test ! -f common/gst-autogen.sh;
 then
   echo There is something wrong with your source tree.
   echo You are missing common/gst-autogen.sh
@@ -30,11 +30,11 @@ CONFIGURE_DEF_OPT='--enable-maintainer-mode --enable-plugin-builddir --enable-de
 autogen_options $@
 
 echo -n "+ check for build tools"
-if test ! -z $NOCHECK; then echo " skipped"; else  echo; fi
+if test ! -z "$NOCHECK"; then echo " skipped"; else  echo; fi
 version_check "autoconf" "$AUTOCONF autoconf autoconf-2.54 autoconf-2.53 autoconf-2.52" \
               "ftp://ftp.gnu.org/pub/gnu/autoconf/" 2 52 || DIE=1
 version_check "automake" "$AUTOMAKE automake automake-1.7 automake-1.6 automake-1.5" \
-              "ftp://ftp.gnu.org/pub/gnu/automake/" 1 5 || DIE=1
+              "ftp://ftp.gnu.org/pub/gnu/automake/" 1 6 || DIE=1
 version_check "libtoolize" "$LIBTOOLIZE libtoolize libtoolize14" \
               "ftp://ftp.gnu.org/pub/gnu/libtool/" 1 4 0 || DIE=1
 version_check "pkg-config" "" \
