@@ -17,22 +17,22 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <string.h>
 #include <sys/time.h>
 
 #include "gstaasink.h"
+#include <gst/video/video.h>
 
 /* elementfactory information */
 static GstElementDetails gst_aasink_details = {
-  "Video sink",
+  "AA sink",
   "Sink/Video",
-  "GPL",
   "An ASCII art videosink",
-  VERSION,
-  "Wim Taymans <wim.taymans@chello.be>",
-  "(C) 2001",
+  "Wim Taymans <wim.taymans@chello.be>"
 };
 
 /* aasink signals and args */
@@ -62,18 +62,20 @@ GST_PAD_TEMPLATE_FACTORY (sink_template,
   "sink",
   GST_PAD_SINK,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
+  gst_caps_new (
     "aasink_caps",
-    "video/raw",
-      "format", 	GST_PROPS_FOURCC (GST_STR_FOURCC ("I420"))
+    "video/x-raw-yuv",
+      GST_VIDEO_YUV_PAD_TEMPLATE_PROPS (
+	      GST_PROPS_FOURCC (GST_STR_FOURCC ("I420")))
   )
 )
-    
+
+static void	gst_aasink_base_init	(gpointer g_class);
 static void	gst_aasink_class_init	(GstAASinkClass *klass);
 static void	gst_aasink_init		(GstAASink *aasink);
 
 static void 	gst_aasink_set_clock 	(GstElement *element, GstClock *clock);
-static void	gst_aasink_chain	(GstPad *pad, GstBuffer *buf);
+static void	gst_aasink_chain	(GstPad *pad, GstData *_data);
 
 static void	gst_aasink_set_property	(GObject *object, guint prop_id, 
 					 const GValue *value, GParamSpec *pspec);
@@ -93,7 +95,7 @@ gst_aasink_get_type (void)
   if (!aasink_type) {
     static const GTypeInfo aasink_info = {
       sizeof(GstAASinkClass),
-      NULL,
+      gst_aasink_base_init,
       NULL,
       (GClassInitFunc) gst_aasink_class_init,
       NULL,
@@ -169,6 +171,16 @@ gst_aasink_dither_get_type (void)
   return dither_type;
 }
 
+static void
+gst_aasink_base_init (gpointer g_class)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+
+  gst_element_class_add_pad_template (element_class,
+		GST_PAD_TEMPLATE_GET (sink_template));
+  gst_element_class_set_details (element_class, &gst_aasink_details);
+}
+
 static void
 gst_aasink_class_init (GstAASinkClass *klass)
 {
@@ -235,7 +247,6 @@ static GstPadLinkReturn
 gst_aasink_sinkconnect (GstPad *pad, GstCaps *caps)
 {
   GstAASink *aasink;
-  gulong print_format;
 
   aasink = GST_AASINK (gst_pad_get_parent (pad));
 
@@ -246,11 +257,10 @@ gst_aasink_sinkconnect (GstPad *pad, GstCaps *caps)
   gst_caps_get_int (caps, "height", &aasink->height);
 
   /* FIXME aasink->format is never set */
-  print_format = GULONG_FROM_LE (aasink->format);
- 	 
-  GST_DEBUG (0, "aasink: setting %08lx (%4.4s)",
-             aasink->format, (gchar*)&print_format);
 
+  GST_DEBUG ("aasink: setting %08lx (" GST_FOURCC_FORMAT ")",
+		 aasink->format, GST_FOURCC_ARGS(aasink->format));
+  
   g_signal_emit( G_OBJECT (aasink), gst_aasink_signals[SIGNAL_HAVE_SIZE], 0,
 		 aasink->width, aasink->height);
 
@@ -330,8 +340,9 @@ gst_aasink_scale (GstAASink *aasink, gchar *src, gchar *dest,
 }
 
 static void
-gst_aasink_chain (GstPad *pad, GstBuffer *buf)
+gst_aasink_chain (GstPad *pad, GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstAASink *aasink;
 
   g_return_if_fail (pad != NULL);
@@ -348,7 +359,7 @@ gst_aasink_chain (GstPad *pad, GstBuffer *buf)
   		    aa_imgwidth (aasink->context),	/* dw */
   		    aa_imgheight (aasink->context));	/* dh */
 
-  GST_DEBUG (0,"videosink: clock wait: %" G_GUINT64_FORMAT, GST_BUFFER_TIMESTAMP(buf));
+  GST_DEBUG ("videosink: clock wait: %" G_GUINT64_FORMAT, GST_BUFFER_TIMESTAMP(buf));
 
   if (aasink->clock) {
     GstClockID id = gst_clock_new_single_shot_id (aasink->clock, GST_BUFFER_TIMESTAMP(buf));
@@ -530,26 +541,22 @@ gst_aasink_change_state (GstElement *element)
 }
 
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-
-  /* create an elementfactory for the aasink element */
-  factory = gst_element_factory_new("aasink",GST_TYPE_AASINK,
-                                   &gst_aasink_details);
-  g_return_val_if_fail(factory != NULL, FALSE);
-
-  gst_element_factory_add_pad_template (factory, 
-		  GST_PAD_TEMPLATE_GET (sink_template));
-
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
+  if (!gst_element_register (plugin, "aasink", GST_RANK_NONE, GST_TYPE_AASINK))
+    return FALSE;
 
   return TRUE;
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "aasink",
-  plugin_init
-};
+  "ASCII Art video sink",
+  plugin_init,
+  VERSION,
+  "GPL",
+  GST_PACKAGE,
+  GST_ORIGIN
+)
