@@ -20,6 +20,9 @@
 
 /* #define GST_DEBUG_ENABLED */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -44,11 +47,8 @@
 static GstElementDetails cdparanoia_details = {
   "CD Audio (cdda) Source, Paranoia IV",
   "Source/File",
-  "LGPL",
   "Read audio from CD in paranoid mode",
-  VERSION,
   "Erik Walthinsen <omega@cse.ogi.edu>",
-  "(C) 2000",
 };
 
 GST_PAD_TEMPLATE_FACTORY (cdparanoia_src_factory,
@@ -57,9 +57,7 @@ GST_PAD_TEMPLATE_FACTORY (cdparanoia_src_factory,
   GST_PAD_ALWAYS,
   GST_CAPS_NEW (
     "cdparanoia_src",
-    "audio/raw",
-	"format", 	GST_PROPS_STRING ("int"),
-	"law", 		GST_PROPS_INT (0),
+    "audio/x-raw-int",
 	"endianness", 	GST_PROPS_INT (G_BYTE_ORDER),
 	"signed", 	GST_PROPS_BOOLEAN (TRUE),
 	"width", 	GST_PROPS_INT (16),
@@ -135,7 +133,7 @@ enum
   ARG_DISCID
 };
 
-
+static void     cdparanoia_base_init (gpointer g_class);
 static void 		cdparanoia_class_init 		(CDParanoiaClass *klass);
 static void 		cdparanoia_init 		(CDParanoia *cdparanoia);
 
@@ -144,7 +142,7 @@ static void 		cdparanoia_set_property 	(GObject *object, guint prop_id,
 static void 		cdparanoia_get_property 	(GObject *object, guint prop_id,
 							 GValue *value, GParamSpec *pspec);
 
-static GstBuffer*	cdparanoia_get 			(GstPad *pad);
+static GstData*	cdparanoia_get 			(GstPad *pad);
 static gboolean 	cdparanoia_event 		(GstPad *pad, GstEvent *event);
 static const GstEventMask*
 			cdparanoia_get_event_mask 	(GstPad *pad);
@@ -179,7 +177,8 @@ cdparanoia_get_type (void)
 
   if (!cdparanoia_type) {
     static const GTypeInfo cdparanoia_info = {
-      sizeof (CDParanoiaClass), NULL,
+      sizeof (CDParanoiaClass),
+			cdparanoia_base_init,
       NULL,
       (GClassInitFunc) cdparanoia_class_init,
       NULL,
@@ -198,6 +197,15 @@ cdparanoia_get_type (void)
   return cdparanoia_type;
 }
 
+static void
+cdparanoia_base_init (gpointer g_class)
+{
+	GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+
+	gst_element_class_add_pad_template (element_class, GST_PAD_TEMPLATE_GET (cdparanoia_src_factory));
+	gst_element_class_set_details (element_class, &cdparanoia_details);
+}
+
 static void
 cdparanoia_class_init (CDParanoiaClass *klass)
 {
@@ -425,7 +433,7 @@ cdparanoia_get_property (GObject *object, guint prop_id, GValue *value, GParamSp
        * See cdparanoia/interface/common-interface.c:FixupTOC
        */
       if (src->d && src->d->cd_extra)
-	g_warning("DiscID on multisession discs might be wrong. Use at own risk.");
+	g_warning("DiscID on multisession discs might be broken. Use at own risk.");
       g_value_set_string (value, src->discid);
       break;
     default:
@@ -439,7 +447,7 @@ cdparanoia_callback (long inpos, int function)
 {
 }
 
-static GstBuffer *
+static GstData *
 cdparanoia_get (GstPad *pad)
 {
   CDParanoia *src;
@@ -451,7 +459,7 @@ cdparanoia_get (GstPad *pad)
 
   /* stop things apropriatly */
   if (src->cur_sector > src->segment_end_sector) {
-    GST_DEBUG (0, "setting EOS");
+    GST_DEBUG ("setting EOS");
 
     buf = GST_BUFFER (gst_event_new (GST_EVENT_EOS));
     gst_element_set_eos (GST_ELEMENT (src));
@@ -482,7 +490,7 @@ cdparanoia_get (GstPad *pad)
   }
 
   /* we're done, push the buffer off now */
-  return buf;
+  return GST_DATA (buf);
 }
 
 /* need some stuff to get a discid (cdparanoia doesn't do cddb but lets
@@ -585,7 +593,7 @@ add_index_associations (CDParanoia *src)
 															 sector_format, sector,
 															 GST_FORMAT_TIME, (gint64) (((CD_FRAMESIZE_RAW >> 2) * sector * GST_SECOND) / 44100),
 															 GST_FORMAT_BYTES, (gint64) (sector << 2),
-															 GST_FORMAT_UNITS, (gint64) ((CD_FRAMESIZE_RAW >> 2) * sector),
+															 GST_FORMAT_DEFAULT, (gint64) ((CD_FRAMESIZE_RAW >> 2) * sector),
 															 NULL);
 #if 0
 		g_print ("Added association for track %d\n", i + 1);
@@ -607,7 +615,7 @@ cdparanoia_open (CDParanoia *src)
 
   g_return_val_if_fail (!GST_FLAG_IS_SET (src, CDPARANOIA_OPEN), FALSE);
 
-  GST_DEBUG_ENTER ("(\"%s\",...)", gst_element_get_name (GST_ELEMENT (src)));
+  GST_DEBUG_OBJECT (src, "trying to open device...");
 
   /* find the device */
   if (src->generic_device != NULL) {
@@ -622,7 +630,7 @@ cdparanoia_open (CDParanoia *src)
 
   /* fail if the device couldn't be found */
   if (src->d == NULL) {
-    GST_DEBUG (0, "couldn't open device");
+    GST_DEBUG ("couldn't open device");
     return FALSE;
   }
 
@@ -637,7 +645,7 @@ cdparanoia_open (CDParanoia *src)
 
   /* open the disc */
   if (cdda_open (src->d)) {
-    GST_DEBUG (0, "couldn't open disc");
+    GST_DEBUG ("couldn't open disc");
     cdda_close (src->d);
     src->d = NULL;
     return FALSE;
@@ -676,7 +684,7 @@ cdparanoia_open (CDParanoia *src)
   /* create the paranoia struct and set it up */
   src->p = paranoia_init (src->d);
   if (src->p == NULL) {
-    GST_DEBUG (0, "couldn't create paranoia struct");
+    GST_DEBUG ("couldn't create paranoia struct");
     return FALSE;
   }
 
@@ -692,7 +700,7 @@ cdparanoia_open (CDParanoia *src)
 
   src->cur_sector = src->first_sector;
   paranoia_seek (src->p, src->cur_sector, SEEK_SET);
-  GST_DEBUG (0, "successfully seek'd to beginning of disk");
+  GST_DEBUG ("successfully seek'd to beginning of disk");
 
   GST_FLAG_SET (src, CDPARANOIA_OPEN);
 
@@ -700,7 +708,7 @@ cdparanoia_open (CDParanoia *src)
 		add_index_associations (src);
 	}
 	
-  GST_DEBUG_LEAVE ("");
+  GST_DEBUG_OBJECT (src, "device successfully openend");
 
   return TRUE;
 }
@@ -734,22 +742,23 @@ cdparanoia_change_state (GstElement *element)
 
   switch (GST_STATE_TRANSITION (element)) {
 	case GST_STATE_NULL_TO_READY:
-		break;
-	case GST_STATE_READY_TO_PAUSED:
 		if (!cdparanoia_open (CDPARANOIA (element))) {
 			g_warning ("cdparanoia: failed opening cd");
 			return GST_STATE_FAILURE;
 		}
 		cdparanoia->seq = 0;
 		break;
+	case GST_STATE_READY_TO_PAUSED:
+		break;
 	case GST_STATE_PAUSED_TO_PLAYING:
 		break;
 	case GST_STATE_PLAYING_TO_PAUSED:
 		break;
 	case GST_STATE_PAUSED_TO_READY:
-		cdparanoia_close (CDPARANOIA (element));
+	        /* FIXME: Need code here to reset element to start of cd */
 		break;
 	case GST_STATE_READY_TO_NULL:
+		cdparanoia_close (CDPARANOIA (element));
 		break;
 	default:
 		break;
@@ -817,7 +826,7 @@ cdparanoia_event (GstPad *pad, GstEvent *event)
       }
       
       if (!res) {
-				GST_DEBUG (0, "could not convert offsets to sectors");
+				GST_DEBUG ("could not convert offsets to sectors");
 				goto error;
       }
       
@@ -850,7 +859,7 @@ cdparanoia_event (GstPad *pad, GstEvent *event)
 																	src->first_sector, src->last_sector);
 				
         if (paranoia_seek (src->p, seg_start_sector, SEEK_SET) > -1) {
-					GST_DEBUG (0, "seeked to %lld", seg_start_sector);
+					GST_DEBUG ("seeked to %" G_GINT64_FORMAT, seg_start_sector);
 					
 					src->segment_start_sector = seg_start_sector;;
 					src->cur_sector = src->segment_start_sector;
@@ -864,7 +873,7 @@ cdparanoia_event (GstPad *pad, GstEvent *event)
 																src->first_sector, src->last_sector);
         src->segment_end_sector = seg_end_sector;;
       }
-      GST_DEBUG (GST_CAT_PLUGIN_INFO, "configured for %d -> %d sectors\n", 
+      GST_DEBUG ("configured for %d -> %d sectors\n", 
 								 src->segment_start_sector, 
 								 src->segment_end_sector);
       break;
@@ -888,7 +897,7 @@ cdparanoia_get_formats (GstPad *pad)
   static GstFormat formats[] = {
     GST_FORMAT_TIME,
     GST_FORMAT_BYTES,
-    GST_FORMAT_UNITS,
+    GST_FORMAT_DEFAULT,
     0,			/* filled later */
     0,			/* filled later */
     0
@@ -918,7 +927,7 @@ cdparanoia_convert (GstPad *pad,
 		switch (*dest_format) {
 		case GST_FORMAT_BYTES:
 			src_value <<= 2;	/* 4 bytes per sample */
-		case GST_FORMAT_UNITS:
+		case GST_FORMAT_DEFAULT:
 			*dest_value = src_value * 44100 / GST_SECOND;
 			break;
 		default:
@@ -939,7 +948,7 @@ cdparanoia_convert (GstPad *pad,
 		break;
 	case GST_FORMAT_BYTES:
 		src_value >>= 2;
-	case GST_FORMAT_UNITS:
+	case GST_FORMAT_DEFAULT:
 		switch (*dest_format) {
 		case GST_FORMAT_BYTES:
 			*dest_value = src_value * 4;
@@ -987,7 +996,7 @@ cdparanoia_convert (GstPad *pad,
 			break;
 		case GST_FORMAT_BYTES:
 			sector <<= 2;
-		case GST_FORMAT_UNITS:
+		case GST_FORMAT_DEFAULT:
 			*dest_value = (CD_FRAMESIZE_RAW >> 2) * sector;
 			break;
 		default:
@@ -1097,29 +1106,21 @@ cdparanoia_get_index (GstElement *element)
 }
 											
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-
-  /* create an elementfactory for the cdparanoia element */
-  factory = gst_element_factory_new ("cdparanoia", 
-		                     GST_TYPE_CDPARANOIA, 
-				     &cdparanoia_details);
-  g_return_val_if_fail (factory != NULL, FALSE);
-
-  /* register the source's caps */
-  gst_element_factory_add_pad_template (factory, 
-		  GST_PAD_TEMPLATE_GET (cdparanoia_src_factory));
-
-  /* and add the cdparanoia element factory to the plugin */
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
+	if (!gst_element_register (plugin, "cdparanoia", GST_RANK_NONE, GST_TYPE_CDPARANOIA))
+		return FALSE;
 
   return TRUE;
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "cdparanoia",
-  plugin_init
-};
+	"Read audio from CD in paranoid mode",
+  plugin_init,
+	VERSION,
+	"GPL",
+	GST_PACKAGE,
+	GST_ORIGIN)
