@@ -20,6 +20,7 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -29,8 +30,6 @@
 #include <assert.h>
 #include <gst/gst.h>
 
-#include "config.h"
-
 #include <dvdnav/dvdnav.h>
 #include <dvdread/nav_print.h>
 
@@ -81,11 +80,8 @@ struct _DVDNavSrcClass {
 GstElementDetails dvdnavsrc_details = {
   "DVD Source",
   "Source/File/DVD",
-  "GPL",
   "Access a DVD with navigation features using libdvdnav",
-  VERSION,
   "David I. Lehn <dlehn@users.sourceforge.net>",
-  "(C) 2002",
 };
 
 
@@ -117,6 +113,7 @@ typedef enum {
 
 
 GType			dvdnavsrc_get_type	(void);
+static void             dvdnavsrc_base_init     (gpointer g_class);
 static void 		dvdnavsrc_class_init	(DVDNavSrcClass *klass);
 static void 		dvdnavsrc_init		(DVDNavSrc *dvdnavsrc);
 
@@ -169,7 +166,8 @@ dvdnavsrc_get_type (void)
 
   if (!dvdnavsrc_type) {
     static const GTypeInfo dvdnavsrc_info = {
-      sizeof(DVDNavSrcClass),      NULL,
+      sizeof(DVDNavSrcClass),
+      dvdnavsrc_base_init,
       NULL,
       (GClassInitFunc)dvdnavsrc_class_init,
       NULL,
@@ -188,6 +186,14 @@ dvdnavsrc_get_type (void)
   return dvdnavsrc_type;
 }
 
+static void
+dvdnavsrc_base_init (gpointer g_class)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+
+  gst_element_class_set_details (element_class, &dvdnavsrc_details);
+}
+
 static void
 dvdnavsrc_class_init (DVDNavSrcClass *klass) 
 {
@@ -1444,23 +1450,21 @@ dvdnavsrc_query (GstPad *pad, GstQueryType type,
 }
 
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-
-  /* create an elementfactory for the dvdnavsrc element */
-  factory = gst_element_factory_new ("dvdnavsrc", GST_TYPE_DVDNAVSRC,
-                                    &dvdnavsrc_details);
-  g_return_val_if_fail (factory != NULL, FALSE);
-  
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
+  if (!gst_element_register (plugin, "dvdnavsrc", GST_RANK_NONE, GST_TYPE_DVDNAVSRC))
+    return FALSE;
   
   return TRUE;
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "dvdnavsrc",
-  plugin_init
-};
+  "Access a DVD with navigation features using libdvdnav",  
+  plugin_init,
+  VERSION,
+  "GPL",
+  GST_PACKAGE,
+  GST_ORIGIN)
