@@ -111,14 +111,16 @@ gst_ffmpeg_codecid_to_caps (enum CodecID    codec_id,
     case CODEC_ID_MP2:
       caps = GST_FF_AUD_CAPS_NEW ("ffmpeg_mp2",
                                   "audio/mpeg",
-                                    "layer", GST_PROPS_INT (2)
+				    "mpegversion", GST_PROPS_INT (1),
+                                    "layer",       GST_PROPS_INT (2)
                                  );
       break;
 
     case CODEC_ID_MP3LAME:
       caps = GST_FF_AUD_CAPS_NEW ("ffmpeg_mp3",
                                   "audio/mpeg",
-                                    "layer", GST_PROPS_INT (3)
+				    "mpegversion", GST_PROPS_INT (1),
+                                    "layer",       GST_PROPS_INT (3)
                                  );
       break;
 
@@ -521,12 +523,12 @@ gst_ffmpeg_pixfmt_to_caps (enum PixelFormat  pix_fmt,
       /* .. */
       break;
     case PIX_FMT_RGBA32:
-      bpp = depth = 32;
+      bpp = 32; depth = 24;
       endianness = G_BIG_ENDIAN;
 #if (G_BYTE_ORDER == G_BIG_ENDIAN)
-      r_mask = 0xff000000; g_mask = 0x00ff0000; b_mask = 0x0000ff00;
+      r_mask = 0x00ff0000; g_mask = 0x0000ff00; b_mask = 0x000000ff;
 #else 
-      r_mask = 0x000000ff; g_mask = 0x0000ff00; b_mask = 0x00ff0000;
+      r_mask = 0x0000ff00; g_mask = 0x00ff0000; b_mask = 0xff000000;
 #endif
       break;
     case PIX_FMT_YUV410P:
@@ -835,73 +837,71 @@ gst_ffmpeg_caps_to_pixfmt (GstCaps        *caps,
     context->frame_rate_base = DEFAULT_FRAME_RATE_BASE;
   }
 
-  if (gst_caps_has_property_typed (caps, "format",
+  if (strcmp (gst_caps_get_mime (caps), "video/x-raw-yuv") == 0) {
+    if (gst_caps_has_property_typed (caps, "format",
 				   GST_PROPS_FOURCC_TYPE)) {
-    guint32 fourcc;
-    gst_caps_get_fourcc_int (caps, "format", &fourcc);
+      guint32 fourcc;
+      gst_caps_get_fourcc_int (caps, "format", &fourcc);
 
-    switch (fourcc) {
-      case GST_MAKE_FOURCC ('Y','U','Y','2'):
-        context->pix_fmt = PIX_FMT_YUV422;
-        break;
-      case GST_MAKE_FOURCC ('I','4','2','0'):
-        context->pix_fmt = PIX_FMT_YUV420P;
-        break;
-      case GST_MAKE_FOURCC ('Y','4','1','B'):
-        context->pix_fmt = PIX_FMT_YUV411P;
-        break;
-      case GST_MAKE_FOURCC ('Y','U','V','9'):
-        context->pix_fmt = PIX_FMT_YUV410P;
-        break;
+      switch (fourcc) {
+	case GST_MAKE_FOURCC ('Y','U','Y','2'):
+	  context->pix_fmt = PIX_FMT_YUV422;
+	  break;
+	case GST_MAKE_FOURCC ('I','4','2','0'):
+	  context->pix_fmt = PIX_FMT_YUV420P;
+	  break;
+	case GST_MAKE_FOURCC ('Y','4','1','B'):
+	  context->pix_fmt = PIX_FMT_YUV411P;
+	  break;
+	case GST_MAKE_FOURCC ('Y','4','2','B'):
+	  context->pix_fmt = PIX_FMT_YUV422P;
+	  break;
+	case GST_MAKE_FOURCC ('Y','U','V','9'):
+	  context->pix_fmt = PIX_FMT_YUV410P;
+	  break;
 #if 0
-      case FIXME:
-        context->pix_fmt = PIX_FMT_YUV444P;
-        break;
+	case FIXME:
+	  context->pix_fmt = PIX_FMT_YUV444P;
+	  break;
 #endif
-      case GST_MAKE_FOURCC ('R','G','B',' '):
-        if (gst_caps_has_property_typed (caps, "depth",
-					 GST_PROPS_INT_TYPE) &&
-	    gst_caps_has_property_typed (caps, "endianness",
-					 GST_PROPS_INT_TYPE)) {
-          gint depth = 0, endianness = 0;
-          gst_caps_get_int (caps, "depth", &depth);
-          gst_caps_get_int (caps, "endianness", &endianness);
+      }
+    }
+  } else if (strcmp (gst_caps_get_mime (caps), "video/x-raw-rgb") == 0) {
+    if (gst_caps_has_property_typed (caps, "bpp", GST_PROPS_INT_TYPE) &&
+	gst_caps_has_property_typed (caps, "red_mask", GST_PROPS_INT_TYPE)) {
+      gint bpp = 0, rmask = 0, endianness = 0;
 
-          switch (depth) {
-            case 32:
-              if (endianness == G_BYTE_ORDER)
-                context->pix_fmt = PIX_FMT_RGBA32;
-              break;
-            case 24:
-              switch (endianness) {
-                case G_LITTLE_ENDIAN:
-                  context->pix_fmt = PIX_FMT_BGR24;
-                  break;
-                case G_BIG_ENDIAN:
-                  context->pix_fmt = PIX_FMT_RGB24;
-                  break;
-                default:
-                  /* nothing */
-                  break;
-              }
-              break;
-            case 16:
-              if (endianness == G_BYTE_ORDER)
-                context->pix_fmt = PIX_FMT_RGB565;
-              break;
-            case 15:
-              if (endianness == G_BYTE_ORDER)
-                context->pix_fmt = PIX_FMT_RGB555;
-              break;
-            default:
-              /* nothing */
-              break;
-          }
-        }
-        break;
-      default:
-        /* nothing */
-        break;
+      gst_caps_get (caps, "bpp",        &bpp,
+			  "red_mask",   &rmask,
+			  "endianness", &endianness, NULL);
+
+      switch (bpp) {
+        case 32:
+#if (G_BYTE_ORDER == G_BIG_ENDIAN)
+          if (rmask == 0x00ff0000)
+#else
+          if (rmask == 0x0000ff00)
+#endif
+            context->pix_fmt = PIX_FMT_RGBA32;
+          break;
+        case 24:
+          if (rmask == 0x0000FF)
+            context->pix_fmt = PIX_FMT_BGR24;
+          else
+            context->pix_fmt = PIX_FMT_RGB24;
+	  break;
+        case 16:
+          if (endianness == G_BYTE_ORDER)
+            context->pix_fmt = PIX_FMT_RGB565;
+	  break;
+        case 15:
+          if (endianness == G_BYTE_ORDER)
+            context->pix_fmt = PIX_FMT_RGB555;
+	  break;
+        default:
+          /* nothing */
+	  break;
+      }
     }
   }
 }
@@ -968,12 +968,12 @@ gst_ffmpeg_formatid_to_caps (const gchar *format_name)
                         );
   } else if (!strcmp (format_name, "asf")) {
     caps = GST_CAPS_NEW ("ffmpeg_asf",
-			 "video/x-asf",
+			 "video/x-ms-asf",
                            NULL
                         );
   } else if (!strcmp (format_name, "avi")) {
     caps = GST_CAPS_NEW ("ffmpeg_avi",
-			 "video/avi",
+			 "video/x-msvideo",
                            NULL
                         );
   } else if (!strcmp (format_name, "wav")) {
@@ -1032,53 +1032,70 @@ gst_ffmpeg_caps_to_codecid (GstCaps        *caps,
 
   mimetype = gst_caps_get_mime(caps);
 
-  if (!strcmp (mimetype, "video/x-raw-rgb") ||
-      !strcmp (mimetype, "video/x-raw-yuv")) {
+  if (!strcmp (mimetype, "video/x-raw-rgb")) {
+
+    id = CODEC_ID_RAWVIDEO;
+
+    if (context != NULL) {
+      gint bpp = 0, endianness = 0, rmask = 0;
+      enum PixelFormat pix_fmt = -1;
+
+      gst_caps_get (caps, "bpp",        &bpp,
+			  "endianness", &endianness,
+			  "rmask",      &rmask, NULL);
+  
+      switch (bpp) {
+        case 15:
+          if (endianness == G_BYTE_ORDER) {
+            pix_fmt = PIX_FMT_RGB555;
+          }
+          break;
+        case 16:
+          if (endianness == G_BYTE_ORDER) {
+            pix_fmt = PIX_FMT_RGB565;
+          }
+          break;
+        case 24:
+          if (rmask == 0xff0000) {
+            pix_fmt = PIX_FMT_RGB24;
+          } else {
+            pix_fmt = PIX_FMT_BGR24;
+          }
+          break;
+        case 32:
+#if (G_BYTE_ORDER == G_BIG_ENDIAN)
+          if (rmask == 0xff0000) {
+#else
+          if (rmask == 0x0000ff) {
+#endif
+            pix_fmt = PIX_FMT_RGBA32;
+          }
+          break;
+        default:
+          /* ... */
+          break;
+      }
+
+      /* only set if actually recognized! */
+      if (pix_fmt != -1) {
+        video = TRUE;
+        context->pix_fmt = pix_fmt;
+      } else {
+        id = CODEC_ID_NONE;
+      }
+    }
+
+  } else if (!strcmp (mimetype, "video/x-raw-yuv")) {
 
     id = CODEC_ID_RAWVIDEO;
 
     if (context != NULL) {
-      gint depth = 0, endianness = 0;
       guint32 fmt_fcc = 0;
       enum PixelFormat pix_fmt = -1;
 
-      if (gst_caps_has_property (caps, "format"))
-        gst_caps_get_fourcc_int (caps, "format", &fmt_fcc);
-      else
-        fmt_fcc = GST_MAKE_FOURCC ('R','G','B',' ');
+      gst_caps_get_fourcc_int (caps, "format", &fmt_fcc);
 
       switch (fmt_fcc) {
-        case GST_MAKE_FOURCC ('R','G','B',' '):
-          gst_caps_get_int (caps, "endianness", &endianness);
-          gst_caps_get_int (caps, "depth", &depth);
-          switch (depth) {
-            case 15:
-              if (endianness == G_BYTE_ORDER) {
-                pix_fmt = PIX_FMT_RGB555;
-              }
-              break;
-            case 16:
-              if (endianness == G_BYTE_ORDER) {
-                pix_fmt = PIX_FMT_RGB565;
-              }
-              break;
-            case 24:
-              if (endianness == G_BIG_ENDIAN) {
-                pix_fmt = PIX_FMT_RGB24;
-              } else {
-                pix_fmt = PIX_FMT_BGR24;
-              }
-              break;
-            case 32:
-              if (endianness == G_BIG_ENDIAN) {
-                pix_fmt = PIX_FMT_RGBA32;
-              }
-              break;
-            default:
-              /* ... */
-              break;
-          }
-          break;
         case GST_MAKE_FOURCC ('Y','U','Y','2'):
           pix_fmt = PIX_FMT_YUV422;
           break;
@@ -1246,28 +1263,36 @@ gst_ffmpeg_caps_to_codecid (GstCaps        *caps,
 
   } else if (!strcmp(mimetype, "audio/mpeg")) {
 
-    if (gst_caps_has_property (caps, "layer")) {
-      gint layer = 0;
-
-      gst_caps_get_int (caps, "layer", &layer);
-      switch (layer) {
-        case 1:
-        case 2:
-          id = CODEC_ID_MP2;
-          break;
-        case 3:
-          id = CODEC_ID_MP3LAME;
-          break;
-        default:
-          /* ... */
-          break;
-      }
-    } else if (gst_caps_has_property (caps, "mpegversion")) {
+    if (gst_caps_has_property (caps, "mpegversion")) {
       gint mpegversion = 0;
 
       gst_caps_get_int (caps, "mpegversion", &mpegversion);
-      if (mpegversion == 4) {
-        id = CODEC_ID_MPEG4AAC;
+      switch (mpegversion) {
+        case 2: /* ffmpeg uses faad for both... */
+        case 4:
+          id = CODEC_ID_MPEG4AAC;
+          break;
+        case 1:
+          if (gst_caps_has_property (caps, "layer")) {
+            gint layer = 0;
+
+            gst_caps_get_int (caps, "layer", &layer);
+            switch (layer) {
+              case 1:
+              case 2:
+                id = CODEC_ID_MP2;
+                break;
+              case 3:
+                id = CODEC_ID_MP3;
+                break;
+              default:
+                /* ... */
+                break;
+            }
+          }
+        default:
+          /* ... */
+          break;
       }
     }
 
