@@ -55,10 +55,12 @@ struct _GstFFMpegDecClass {
   GstPadTemplate *srctempl, *sinktempl;
 };
 
-typedef struct {
+typedef struct _GstFFMpegDecClassParams GstFFMpegDecClassParams;
+
+struct _GstFFMpegDecClassParams {
   AVCodec *in_plugin;
-  GstPadTemplate *srctempl, *sinktempl;
-} GstFFMpegDecClassParams;
+  GstCaps *srccaps, *sinkcaps;
+};
 
 #define GST_TYPE_FFMPEGDEC \
   (gst_ffmpegdec_get_type())
@@ -84,6 +86,7 @@ enum {
 static GHashTable *global_plugins;
 
 /* A number of functon prototypes are given so we can refer to them later. */
+static void	gst_ffmpegdec_base_init		(GstFFMpegDecClass *klass);
 static void	gst_ffmpegdec_class_init	(GstFFMpegDecClass *klass);
 static void	gst_ffmpegdec_init		(GstFFMpegDec *ffmpegdec);
 static void	gst_ffmpegdec_dispose		(GObject      *object);
@@ -96,34 +99,71 @@ static void	gst_ffmpegdec_chain		(GstPad    *pad,
 static GstElementStateReturn
 		gst_ffmpegdec_change_state	(GstElement *element);
 
+#if 0
 /* some sort of bufferpool handling, but different */
 static int	gst_ffmpegdec_get_buffer	(AVCodecContext *context,
 						 AVFrame        *picture);
 static void	gst_ffmpegdec_release_buffer	(AVCodecContext *context,
 						 AVFrame        *picture);
+#endif
 
 static GstElementClass *parent_class = NULL;
 
 /*static guint gst_ffmpegdec_signals[LAST_SIGNAL] = { 0 }; */
 
+static void
+gst_ffmpegdec_base_init (GstFFMpegDecClass *klass)
+{
+  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+  GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
+  GstFFMpegDecClassParams *params;
+  GstElementDetails *details;
+  GstPadTemplate *sinktempl, *srctempl;
+
+  params = g_hash_table_lookup (global_plugins,
+		GINT_TO_POINTER (G_OBJECT_CLASS_TYPE (gobject_class)));
+  if (!params)
+    params = g_hash_table_lookup (global_plugins,
+		GINT_TO_POINTER (0));
+  g_assert (params);
+
+  /* construct the element details struct */
+  details = g_new0 (GstElementDetails, 1);
+  details->longname = g_strdup_printf("FFMPEG %s decoder",
+				      params->in_plugin->name);
+  details->klass = g_strdup_printf("Codec/Decoder/%s",
+				   (params->in_plugin->type == CODEC_TYPE_VIDEO) ?
+				   "Video" : "Audio");
+  details->description = g_strdup_printf("FFMPEG %s decoder",
+					 params->in_plugin->name);
+  details->author = g_strdup("Wim Taymans <wim.taymans@chello.be>\n"
+			     "Ronald Bultje <rbultje@ronald.bitfreak.net>");
+
+  /* pad templates */
+  sinktempl = gst_pad_template_new ("sink", GST_PAD_SINK,
+				    GST_PAD_ALWAYS, params->sinkcaps, NULL);
+  srctempl = gst_pad_template_new ("src", GST_PAD_SRC,
+				   GST_PAD_ALWAYS, params->srccaps, NULL);
+
+  gst_element_class_add_pad_template (element_class, srctempl);
+  gst_element_class_add_pad_template (element_class, sinktempl);
+  gst_element_class_set_details (element_class, details);
+
+  klass->in_plugin = params->in_plugin;
+  klass->srctempl = srctempl;
+  klass->sinktempl = sinktempl;
+}
+
 static void
 gst_ffmpegdec_class_init (GstFFMpegDecClass *klass)
 {
   GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
-  GstFFMpegDecClassParams *params;
 
   gobject_class = (GObjectClass*)klass;
   gstelement_class = (GstElementClass*)klass;
 
-  parent_class = g_type_class_ref(GST_TYPE_ELEMENT);
-
-  params = g_hash_table_lookup (global_plugins,
-		  GINT_TO_POINTER (G_OBJECT_CLASS_TYPE (gobject_class)));
-
-  klass->in_plugin = params->in_plugin;
-  klass->srctempl = params->srctempl;
-  klass->sinktempl = params->sinktempl;
+  parent_class = g_type_class_peek_parent (klass);
 
   gobject_class->dispose = gst_ffmpegdec_dispose;
   gstelement_class->change_state = gst_ffmpegdec_change_state;
@@ -154,11 +194,10 @@ static void
 gst_ffmpegdec_dispose (GObject *object)
 {
   GstFFMpegDec *ffmpegdec = (GstFFMpegDec *) object;
-  /* close old session */
-  if (ffmpegdec->opened) {
-    avcodec_close (ffmpegdec->context);
-    ffmpegdec->opened = FALSE;
-  }
+
+  G_OBJECT_CLASS (parent_class)->dispose (object);
+  /* old session should have been closed in element_class->dispose */
+  g_assert (!ffmpegdec->opened);
 
   /* clean up remaining allocated data */
   av_free (ffmpegdec->context);
@@ -185,9 +224,11 @@ gst_ffmpegdec_connect (GstPad  *pad,
   /* set defaults */
   avcodec_get_context_defaults (ffmpegdec->context);
 
+#if 0
   /* set buffer functions */
   ffmpegdec->context->get_buffer = gst_ffmpegdec_get_buffer;
   ffmpegdec->context->release_buffer = gst_ffmpegdec_release_buffer;
+#endif
 
   /* get size and so */
   gst_ffmpeg_caps_to_codectype (oclass->in_plugin->type,
@@ -216,6 +257,7 @@ gst_ffmpegdec_connect (GstPad  *pad,
   return GST_PAD_LINK_OK;
 }
 
+#if 0
 static int
 gst_ffmpegdec_get_buffer (AVCodecContext *context,
 			  AVFrame        *picture)
@@ -268,6 +310,7 @@ gst_ffmpegdec_release_buffer (AVCodecContext *context,
     picture->linesize[i] = 0;
   }
 }
+#endif
 
 static void
 gst_ffmpegdec_chain (GstPad    *pad,
@@ -301,13 +344,38 @@ gst_ffmpegdec_chain (GstPad    *pad,
 
     switch (oclass->in_plugin->type) {
       case CODEC_TYPE_VIDEO:
+	/* workarounds, functions write to buffers:
+	   libavcodec/svq1.c:svq1_decode_frame writes to the given buffer.
+           libavcodec/svq3.c:svq3_decode_slice_header too */
+	if (oclass->in_plugin->id == CODEC_ID_SVQ1 ||
+            oclass->in_plugin->id == CODEC_ID_SVQ3) {
+	  inbuf = gst_buffer_copy_on_write(inbuf);
+	  data = GST_BUFFER_DATA (inbuf);
+	  size = GST_BUFFER_SIZE (inbuf);
+	}
         len = avcodec_decode_video (ffmpegdec->context,
 				    ffmpegdec->picture,
 				    &have_data,
 				    data, size);
         if (have_data) {
-          outbuf = GST_BUFFER (ffmpegdec->picture->base[0]);
-          GST_BUFFER_SIZE (outbuf) = GST_BUFFER_MAXSIZE (outbuf);
+          /* libavcodec constantly crashes on stupid buffer allocation
+           * errors inside. This drives me crazy, so we let it allocate
+           * it's own buffers and copy to our own buffer afterwards... */
+          AVPicture pic;
+          gint size = avpicture_get_size (ffmpegdec->context->pix_fmt,
+					  ffmpegdec->context->width,
+					  ffmpegdec->context->height);
+          outbuf = gst_buffer_new_and_alloc (size);
+          avpicture_fill (&pic, GST_BUFFER_DATA (outbuf),
+			  ffmpegdec->context->pix_fmt,
+			  ffmpegdec->context->width,
+			  ffmpegdec->context->height);
+          img_convert (&pic, ffmpegdec->context->pix_fmt,
+		       (AVPicture *) ffmpegdec->picture,
+		       ffmpegdec->context->pix_fmt,
+		       ffmpegdec->context->width,
+		       ffmpegdec->context->height);
+
           /* this isn't necessarily true, but it's better than nothing */
           GST_BUFFER_DURATION (outbuf) = GST_BUFFER_DURATION (inbuf);
         }
@@ -389,10 +457,9 @@ gst_ffmpegdec_change_state (GstElement *element)
 gboolean
 gst_ffmpegdec_register (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
   GTypeInfo typeinfo = {
     sizeof(GstFFMpegDecClass),      
-    NULL,
+    (GBaseInitFunc)gst_ffmpegdec_base_init,
     NULL,
     (GClassInitFunc)gst_ffmpegdec_class_init,
     NULL,
@@ -402,7 +469,6 @@ gst_ffmpegdec_register (GstPlugin *plugin)
     (GInstanceInitFunc)gst_ffmpegdec_init,
   };
   GType type;
-  GstElementDetails *details;
   AVCodec *in_plugin;
   
   in_plugin = first_avcodec;
@@ -410,15 +476,14 @@ gst_ffmpegdec_register (GstPlugin *plugin)
   global_plugins = g_hash_table_new (NULL, NULL);
 
   while (in_plugin) {
-    gchar *type_name;
-    GstPadTemplate *sinktempl, *srctempl;
-    GstCaps *sinkcaps, *srccaps;
     GstFFMpegDecClassParams *params;
+    GstCaps *srccaps, *sinkcaps;
+    gchar *type_name;
 
     /* no quasi-codecs, please */
     if (in_plugin->id == CODEC_ID_RAWVIDEO ||
 	(in_plugin->id >= CODEC_ID_PCM_S16LE &&
-	 in_plugin->id >= CODEC_ID_PCM_ALAW)) {
+	 in_plugin->id <= CODEC_ID_PCM_ALAW)) {
       goto next;
     }
 
@@ -442,53 +507,27 @@ gst_ffmpegdec_register (GstPlugin *plugin)
       goto next;
     }
 
-    /* create the gtk type now */
-    type = g_type_register_static(GST_TYPE_ELEMENT, type_name , &typeinfo, 0);
-
-    /* construct the element details struct */
-    details = g_new0 (GstElementDetails, 1);
-    details->longname = g_strdup(in_plugin->name);
-    details->klass = g_strdup_printf("Codec/%s/Decoder",
-				     (in_plugin->type == CODEC_TYPE_VIDEO) ?
-				     "Video" : "Audio");
-    details->license = g_strdup("LGPL");
-    details->description = g_strdup_printf("FFMPEG %s decoder",
-					   in_plugin->name);
-    details->version = g_strdup(VERSION);
-    details->author = g_strdup("The FFMPEG crew, "
-				"Wim Taymans <wim.taymans@chello.be>, "
-				"Ronald Bultje <rbultje@ronald.bitfreak.net>");
-    details->copyright = g_strdup("(c) 2001-2003");
-
-    /* register the plugin with gstreamer */
-    factory = gst_element_factory_new(type_name,type,details);
-    g_return_val_if_fail(factory != NULL, FALSE);
-
-    gst_element_factory_set_rank (factory, GST_ELEMENT_RANK_MARGINAL);
-
-    sinktempl = gst_pad_template_new ("sink", GST_PAD_SINK,
-				      GST_PAD_ALWAYS, sinkcaps, NULL);
-    gst_element_factory_add_pad_template (factory, sinktempl);
-
-    srctempl = gst_pad_template_new ("src", GST_PAD_SRC,
-				     GST_PAD_ALWAYS, srccaps, NULL);
-    gst_element_factory_add_pad_template (factory, srctempl);
-
     params = g_new0 (GstFFMpegDecClassParams, 1);
     params->in_plugin = in_plugin;
-    params->sinktempl = sinktempl;
-    params->srctempl = srctempl;
+    params->srccaps = srccaps;
+    params->sinkcaps = sinkcaps;
+    g_hash_table_insert (global_plugins, 
+		         GINT_TO_POINTER (0), 
+			 (gpointer) params);
+    
+    /* create the gtype now */
+    type = g_type_register_static(GST_TYPE_ELEMENT, type_name , &typeinfo, 0);
+    if (!gst_element_register (plugin, type_name, GST_RANK_MARGINAL, type))
+      return FALSE;
 
     g_hash_table_insert (global_plugins, 
 		         GINT_TO_POINTER (type), 
 			 (gpointer) params);
 
-    /* The very last thing is to register the elementfactory with the plugin. */
-    gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
 next:
     in_plugin = in_plugin->next;
   }
+  g_hash_table_remove (global_plugins, GINT_TO_POINTER (0));
 
   return TRUE;
 }
