@@ -22,7 +22,6 @@
 #define __FLACENC_H__
 
 
-#include <config.h>
 #include <gst/gst.h>
 
 #include <FLAC/all.h>
@@ -45,8 +44,6 @@ struct _FlacEnc {
 
   GstPad *sinkpad,*srcpad;
 
-  GstCaps *metadata;
-
   gboolean 	 first;
   GstBuffer	*first_buf;
   gboolean 	 eos;
@@ -60,6 +57,8 @@ struct _FlacEnc {
 
   FLAC__SeekableStreamEncoder *encoder;
   FLAC__StreamMetadata **meta;
+
+  GstTagList *	   tags;
 };
 
 struct _FlacEncClass {
