@@ -17,6 +17,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <string.h>
 
@@ -29,12 +32,9 @@
 /* elementfactory information */
 static GstElementDetails gst_mplex_details = {
   "MPlex multiplexer",
-  "Codec/Audio/Decoder",
-  "GPL",
+  "Codec/Muxer",
   "multiplex mpeg audio and video into a system stream",
-  VERSION,
   "Wim Taymans <wim.taymans@chello.be> ",
-  "(C) 2002",
 };
 
 /* Sidec signals and args */
@@ -87,6 +87,7 @@ GST_PAD_TEMPLATE_FACTORY (audio_sink_factory,
   GST_CAPS_NEW (
     "sink_audio",
     "audio/mpeg",
+      "mpegversion", GST_PROPS_INT (1),
       "layer", GST_PROPS_INT_RANGE (1, 3)
   )
 )
@@ -136,6 +137,7 @@ gst_mplex_mux_format_get_type (void)
   return mplex_mux_format_type;
 }
 
+static void     gst_mplex_base_init             (gpointer g_class);
 static void 	gst_mplex_class_init		(GstMPlex *klass);
 static void 	gst_mplex_init			(GstMPlex *mplex);
 
@@ -166,7 +168,7 @@ gst_mplex_get_type (void)
   if (!mplex_type) {
     static const GTypeInfo mplex_info = {
       sizeof(GstMPlexClass),      
-      NULL,
+      gst_mplex_base_init,
       NULL,
       (GClassInitFunc) gst_mplex_class_init,
       NULL,
@@ -182,6 +184,20 @@ gst_mplex_get_type (void)
   return mplex_type;
 }
 
+static void
+gst_mplex_base_init (gpointer g_class)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+
+  gst_element_class_add_pad_template (element_class, GST_PAD_TEMPLATE_GET (src_factory));
+  gst_element_class_add_pad_template (element_class, GST_PAD_TEMPLATE_GET (audio_sink_factory));
+  gst_element_class_add_pad_template (element_class, GST_PAD_TEMPLATE_GET (video_sink_factory));
+  gst_element_class_add_pad_template (element_class, GST_PAD_TEMPLATE_GET (private_1_sink_factory));
+  gst_element_class_add_pad_template (element_class, GST_PAD_TEMPLATE_GET (private_2_sink_factory));
+
+  gst_element_class_set_details (element_class, &gst_mplex_details);
+}
+
 static void
 gst_mplex_class_init (GstMPlex *klass) 
 {
@@ -573,35 +589,25 @@ gst_mplex_get_property (GObject *object, guint prop_id, GValue *value, GParamSpe
 }
 
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
+  if (!gst_library_load ("gstbytestream"))
+    return FALSE;
 
-  /* this filter needs the bytestream package */
-/*  if (!gst_library_load ("gstbytestream"))
-    return FALSE;*/
-
-  /* create an elementfactory for the avi_demux element */
-  factory = gst_element_factory_new ("mplex",GST_TYPE_MPLEX,
-                                    &gst_mplex_details);
-  g_return_val_if_fail (factory != NULL, FALSE);
-  gst_element_factory_set_rank (factory, GST_ELEMENT_RANK_NONE);
-
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (src_factory));
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (audio_sink_factory));
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (video_sink_factory));
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (private_1_sink_factory));
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (private_2_sink_factory));
-
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
+  if (!gst_element_register (plugin, "mplex", GST_RANK_NONE, GST_TYPE_MPLEX))
+    return FALSE;
 
   return TRUE;
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "mplex",
-  plugin_init
-};
+  "MPlexs an audio and video stream into a system stream",
+  plugin_init,
+  VERSION,
+  "LGPL",
+  GST_PACKAGE,
+  GST_ORIGIN)
 
