@@ -21,7 +21,9 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+#include <gst/riff/riff-ids.h>
 #include "gstasfdemux.h"
+#include "gstasfmux.h" /* for the type registering */
 #include "asfheaders.h"
 
 /* elementfactory information */
@@ -448,7 +450,7 @@ gst_asf_demux_process_segment (GstASFDemux       *asf_demux,
   
   if (replic_size > 1) {
     asf_replicated_data *replicated_data_header;
-    guint8              **replicated_data = NULL;
+    guint8              *replicated_data = NULL;
     guint8              *ptr;
 
     segment_info.compressed = FALSE;
@@ -464,7 +466,7 @@ gst_asf_demux_process_segment (GstASFDemux       *asf_demux,
     segment_info.segment_size = GUINT32_FROM_LE (replicated_data_header->object_size);
 
     if (replic_size > 8) {
-      gst_asf_demux_read_object_header_rest (asf_demux, replicated_data, replic_size - 8);
+      gst_asf_demux_read_object_header_rest (asf_demux, &replicated_data, replic_size - 8);
     }
 
     rsize += replic_size;
@@ -1256,12 +1258,14 @@ gst_asf_demux_audio_caps (guint16 codec_id,
     case GST_RIFF_WAVE_FORMAT_MPEGL3: /* mp3 */
       caps = GST_ASF_AUD_CAPS_NEW ("asf_demux_audio_src_mp3",
 				   "audio/mpeg",
+				     "mpegversion", GST_PROPS_INT (1),
 				     "layer", GST_PROPS_INT (3));
       break;
 
     case GST_RIFF_WAVE_FORMAT_MPEGL12: /* mp1 or mp2 */
       caps = GST_ASF_AUD_CAPS_NEW ("asf_demux_audio_src_mp12",
 				   "audio/mpeg",
+				     "mpegversion", GST_PROPS_INT (1),
 				     "layer", GST_PROPS_INT (2));
       break;
 
@@ -1624,7 +1628,8 @@ plugin_init (GstPlugin *plugin)
     return FALSE;
 
   /* create an elementfactory for the asf_demux element */
-  if (!gst_element_register (plugin, "asfdemux", GST_RANK_NONE, GST_TYPE_ASF_DEMUX))
+  if (!gst_element_register (plugin, "asfdemux", GST_RANK_PRIMARY, GST_TYPE_ASF_DEMUX) ||
+      !gst_element_register (plugin, "asfmux", GST_RANK_NONE, GST_TYPE_ASFMUX))
     return FALSE;
 
   return TRUE;
@@ -1633,12 +1638,11 @@ plugin_init (GstPlugin *plugin)
 GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
-  "asfdemux",
-  "Demuxes ASF streams",
+  "asf",
+  "Demuxes and muxes audio and video in Microsofts ASF format",
   plugin_init,
   VERSION,
   "LGPL",
-  GST_COPYRIGHT,
   GST_PACKAGE,
   GST_ORIGIN
 )
