@@ -19,20 +19,20 @@
 /* based on the Area Based Deinterlacer (for RGB frames)        */
 /* (a VirtualDub filter) from Gunnar Thalin <guth@home.se>      */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <string.h>
 #include <gst/gst.h>
 #include "gstdeinterlace.h"
 
 /* elementfactory information */
-static GstElementDetails deinterlace_details = {
+static GstElementDetails deinterlace_details = GST_ELEMENT_DETAILS (
   "Deinterlace",
-  "Filter/Video",
-  "LGPL",
+  "Filter/Effect/Video",
   "Deinterlace video",
-  VERSION,
-  "Wim Taymans <wim.taymans@chello.be>",
-  "(C) 2001",
-};
+  "Wim Taymans <wim.taymans@chello.be>"
+);
 
 
 /* Filter signals and args */
@@ -55,10 +55,11 @@ GST_PAD_TEMPLATE_FACTORY (deinterlace_src_factory,
   GST_PAD_ALWAYS,
   GST_CAPS_NEW (
    "deinterlace_src",
-   "video/raw",
+   "video/x-raw-yuv",
      "format",   GST_PROPS_FOURCC (GST_STR_FOURCC ("I420")),
      "width",    GST_PROPS_INT_POSITIVE,
-     "height",   GST_PROPS_INT_POSITIVE
+     "height",   GST_PROPS_INT_POSITIVE,
+     "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT)
   )
 )
 
@@ -68,15 +69,17 @@ GST_PAD_TEMPLATE_FACTORY (deinterlace_sink_factory,
   GST_PAD_ALWAYS,
   GST_CAPS_NEW (
    "deinterlace_src",
-   "video/raw",
+   "video/x-raw-yuv",
      "format",   GST_PROPS_FOURCC (GST_STR_FOURCC ("I420")),
      "width",    GST_PROPS_INT_POSITIVE,
-     "height",   GST_PROPS_INT_POSITIVE
+     "height",   GST_PROPS_INT_POSITIVE,
+     "framerate", GST_PROPS_FLOAT_RANGE (0, G_MAXFLOAT)
   )
 )
 
 static GType 		gst_deinterlace_get_type		(void);
 
+static void             gst_deinterlace_base_init               (gpointer g_class);
 static void		gst_deinterlace_class_init		(GstDeInterlaceClass *klass);
 static void		gst_deinterlace_init			(GstDeInterlace *filter);
 
@@ -85,7 +88,7 @@ static void		gst_deinterlace_set_property		(GObject *object, guint prop_id,
 static void		gst_deinterlace_get_property		(GObject *object, guint prop_id, 
 								 GValue *value, GParamSpec *pspec);
 
-static void		gst_deinterlace_chain			(GstPad *pad, GstBuffer *buf);
+static void		gst_deinterlace_chain			(GstPad *pad, GstData *_data);
 
 static GstElementClass *parent_class = NULL;
 /*static guint gst_filter_signals[LAST_SIGNAL] = { 0 }; */
@@ -96,7 +99,8 @@ gst_deinterlace_get_type(void) {
 
   if (!deinterlace_type) {
     static const GTypeInfo deinterlace_info = {
-      sizeof(GstDeInterlaceClass),      NULL,
+      sizeof(GstDeInterlaceClass),      
+      gst_deinterlace_base_init,
       NULL,
       (GClassInitFunc)gst_deinterlace_class_init,
       NULL,
@@ -110,6 +114,19 @@ gst_deinterlace_get_type(void) {
   return deinterlace_type;
 }
 
+static void
+gst_deinterlace_base_init (gpointer g_class)
+{ 
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+
+  gst_element_class_add_pad_template (element_class, 
+		  GST_PAD_TEMPLATE_GET (deinterlace_src_factory));
+  gst_element_class_add_pad_template (element_class, 
+		  GST_PAD_TEMPLATE_GET (deinterlace_sink_factory));
+
+  gst_element_class_set_details (element_class, &deinterlace_details);
+}
+
 static void
 gst_deinterlace_class_init (GstDeInterlaceClass *klass)
 {
@@ -157,7 +174,7 @@ gst_deinterlace_sinkconnect (GstPad *pad, GstCaps *caps)
     filter->picsize = filter->width*filter->height;
     filter->src = g_malloc(filter->picsize);
   }
-  return gst_pad_try_set_caps (filter->srcpad, caps);
+  return gst_pad_try_set_caps (filter->srcpad, gst_caps_ref (caps));
 }
 
 static void
@@ -182,8 +199,9 @@ gst_deinterlace_init (GstDeInterlace *filter)
 }
 
 static void
-gst_deinterlace_chain (GstPad *pad, GstBuffer *buf)
+gst_deinterlace_chain (GstPad *pad, GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstDeInterlace *filter;
   gint y0, y1, y2, y3;
   guchar *psrc1, *psrc2, *psrc3, *pdst1, *yuvptr, *src;
@@ -290,7 +308,7 @@ gst_deinterlace_chain (GstPad *pad, GstBuffer *buf)
     }
   }
 
-  gst_pad_push (filter->srcpad, buf);
+  gst_pad_push (filter->srcpad, GST_DATA (buf));
 }
 
 static void
@@ -352,27 +370,22 @@ gst_deinterlace_get_property (GObject *object, guint prop_id, GValue *value, GPa
 }
 
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-
-  factory = gst_element_factory_new("deinterlace",GST_TYPE_DEINTERLACE,
-                                   &deinterlace_details);
-  g_return_val_if_fail(factory != NULL, FALSE);
-  
-  gst_element_factory_add_pad_template (factory, 
-		  GST_PAD_TEMPLATE_GET (deinterlace_src_factory));
-  gst_element_factory_add_pad_template (factory, 
-		  GST_PAD_TEMPLATE_GET (deinterlace_sink_factory));
-
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
+  if (!gst_element_register (plugin, "deinterlace", GST_RANK_NONE, gst_deinterlace_get_type()))
+    return FALSE;
 
   return TRUE;
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "deinterlace",
-  plugin_init
-};
+  "Deinterlace video",
+  plugin_init,
+  VERSION,
+  "LGPL",
+  GST_PACKAGE,
+  GST_ORIGIN  
+);
