@@ -40,6 +40,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <math.h>
 #include <string.h>
 #include <gst/gst.h>
@@ -82,16 +85,12 @@ struct _GstRevTVClass
 };
 
 /* elementfactory information */
-GstElementDetails gst_revtv_details = {
+static GstElementDetails gst_revtv_details = GST_ELEMENT_DETAILS (
   "RevTV",
-  "Filter/Video/Effect",
-  "LGPL",
+  "Filter/Effect/Video",
   "A video waveform monitor for each line of video processed",
-  VERSION,
-  "Wim Taymans <wim.taymans@chello.be>",
-  "(C) 2001 FUKUCHI Kentarou,"
-  "(c) 2002 Ed Tannenbaum",
-};
+  "Wim Taymans <wim.taymans@chello.be>"
+);
 
 
 /* Filter signals and args */
@@ -109,6 +108,7 @@ enum
   ARG_GAIN,
 };
 
+static void 	gst_revtv_base_init 		(gpointer g_class);
 static void 	gst_revtv_class_init 		(GstRevTVClass * klass);
 static void 	gst_revtv_init 			(GstRevTV * filter);
 
@@ -117,7 +117,7 @@ static void 	gst_revtv_set_property 		(GObject * object, guint prop_id,
 static void 	gst_revtv_get_property 		(GObject * object, guint prop_id,
 					  	 GValue * value, GParamSpec * pspec);
 
-static void 	gst_revtv_chain 		(GstPad * pad, GstBuffer * buf);
+static void 	gst_revtv_chain 		(GstPad * pad, GstData *_data);
 
 static GstElementClass *parent_class = NULL;
 /* static guint gst_revtv_signals[LAST_SIGNAL] = { 0 }; */
@@ -128,7 +128,8 @@ GType gst_revtv_get_type (void)
 
   if (!revtv_type) {
     static const GTypeInfo revtv_info = {
-      sizeof (GstRevTVClass), NULL,
+      sizeof (GstRevTVClass), 
+      gst_revtv_base_init,
       NULL,
       (GClassInitFunc) gst_revtv_class_init,
       NULL,
@@ -143,6 +144,17 @@ GType gst_revtv_get_type (void)
   return revtv_type;
 }
 
+static void
+gst_revtv_base_init (gpointer g_class)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+
+  gst_element_class_add_pad_template (element_class, gst_effectv_src_factory ());
+  gst_element_class_add_pad_template (element_class, gst_effectv_sink_factory ());
+ 
+  gst_element_class_set_details (element_class, &gst_revtv_details);
+}
+
 static void
 gst_revtv_class_init (GstRevTVClass * klass)
 {
@@ -181,7 +193,7 @@ gst_revtv_sinkconnect (GstPad * pad, GstCaps * caps)
   gst_caps_get_int (caps, "width", &filter->width);
   gst_caps_get_int (caps, "height", &filter->height);
 
-  return gst_pad_try_set_caps (filter->srcpad, caps);
+  return gst_pad_try_set_caps (filter->srcpad, gst_caps_ref (caps));
 }
 
 static void
@@ -203,8 +215,9 @@ gst_revtv_init (GstRevTV * filter)
 
 
 static void
-gst_revtv_chain (GstPad * pad, GstBuffer * buf)
+gst_revtv_chain (GstPad * pad, GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstRevTV *filter;
   guint32 *src, *dest;
   GstBuffer *outbuf;
@@ -245,7 +258,7 @@ gst_revtv_chain (GstPad * pad, GstBuffer * buf)
   
   gst_buffer_unref (buf);
 
-  gst_pad_push (filter->srcpad, outbuf);
+  gst_pad_push (filter->srcpad, GST_DATA (outbuf));
 }
 
 static void
