@@ -28,24 +28,12 @@
 
 #define JIFFIE  (GST_SECOND/70)
 
-static GstCaps* flxdec_type_find (GstByteStream *bs, gpointer private);
-
 /* flx element information */
 static GstElementDetails flxdec_details = {
   "FLX Decoder",
-  "Codec/Audio/Decoder",
-  "LGPL",
+  "Codec/Decoder/Audio",
   "FLX decoder",
-  VERSION,
-  "Sepp Wijnands <mrrazz@garbage-coderz.net>",
-  "(C) 2001",
-};
-
-static GstTypeDefinition flxdec_definition = {
-  "flxdec_video/fli",
-  "video/fli",
-  ".flc .fli",
-  flxdec_type_find,
+  "Sepp Wijnands <mrrazz@garbage-coderz.net>"
 };
 
 /* Flx signals and args */
@@ -92,6 +80,7 @@ GST_PAD_TEMPLATE_FACTORY (src_video_factory,
 
 
 static void	gst_flxdec_class_init	(GstFlxDecClass *klass);
+static void	gst_flxdec_base_init	(GstFlxDecClass *klass);
 static void	gst_flxdec_init		(GstFlxDec *flxdec);
 
 static void	gst_flxdec_loop		(GstElement *element);
@@ -112,37 +101,6 @@ static void 	flx_decode_delta_flc	(GstFlxDec *, guchar *, guchar *);
 
 static GstElementClass *parent_class = NULL;
 
-static GstCaps* 
-flxdec_type_find (GstByteStream *bs, gpointer private)
-{
-  GstBuffer *buf = NULL;
-  GstCaps *new = NULL;
-
-  if (gst_bytestream_peek (bs, &buf, 134) == 134) {
-    guint8 *data = GST_BUFFER_DATA (buf);
-
-    /* check magic */
-    if ((data[4] == 0x11 || data[4] == 0x12 ||
-         data[4] == 0x30 || data[4] == 0x44) &&
-        data[5] == 0xaf) {
-      /* check the frame type of the first frame */
-      if ((data[132] == 0x00 || data[132] == 0xfa) && data[133] == 0xf1) {
-        GST_DEBUG ("GstFlxDec: found supported flx format");
-        new = gst_caps_new ("flxdec_type_find",
-			    "video/x-fli",
-			      NULL);
-      }
-    }
-  }
-
-  if (buf != NULL) {
-    gst_buffer_unref (buf);
-  }
-
-  return new;
-}
-
-
 GType
 gst_flxdec_get_type(void) 
 {
@@ -150,7 +108,8 @@ gst_flxdec_get_type(void)
 
   if (!flxdec_type) {
     static const GTypeInfo flxdec_info = {
-      sizeof(GstFlxDecClass),      NULL,
+      sizeof(GstFlxDecClass),
+      (GBaseInitFunc)gst_flxdec_base_init,
       NULL,
       (GClassInitFunc)gst_flxdec_class_init,
       NULL,
@@ -164,6 +123,18 @@ gst_flxdec_get_type(void)
   return flxdec_type;
 }
 
+static void
+gst_flxdec_base_init (GstFlxDecClass *klass)
+{
+  GstElementClass *gstelement_class = GST_ELEMENT_CLASS (klass);
+  
+  gst_element_class_set_details (gstelement_class, &flxdec_details);
+  gst_element_class_add_pad_template (gstelement_class,
+	GST_PAD_TEMPLATE_GET (sink_factory));
+  gst_element_class_add_pad_template (gstelement_class,
+	GST_PAD_TEMPLATE_GET (src_video_factory));
+}
+
 static void 
 gst_flxdec_class_init (GstFlxDecClass *klass) 
 {
@@ -685,29 +656,23 @@ gst_flxdec_get_property (GObject *object, guint prop_id, GValue *value, GParamSp
 }
 
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-  GstTypeFactory *type;
+  if (!gst_library_load ("gstbytestream"))
+    return FALSE;
 
-  factory = gst_element_factory_new("flxdec", GST_TYPE_FLXDEC, &flxdec_details);
-  g_return_val_if_fail(factory != NULL, FALSE);
-  gst_element_factory_set_rank (factory, GST_ELEMENT_RANK_PRIMARY);
-
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (sink_factory));
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (src_video_factory));
-
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
-  type = gst_type_factory_new (&flxdec_definition);
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (type));
-
-  return TRUE;
+  return gst_element_register (plugin, "flxdec",
+			       GST_RANK_PRIMARY, GST_TYPE_FLXDEC);
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "flxdec",
-  plugin_init
-};
+  "FLX video decoder",
+  plugin_init,
+  VERSION,
+  GST_LICENSE,
+  GST_PACKAGE,
+  GST_ORIGIN
+)
