@@ -19,26 +19,31 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include "config.h"
+
 #include "gstfloat2int.h"
 #include "gstint2float.h"
 
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  gboolean ret;
-
-  ret = gst_int2float_factory_init (plugin);
-  if (!ret) return FALSE;
-
-  ret = gst_float2int_factory_init (plugin);
-  if (!ret) return FALSE;
+  if (!gst_element_register (plugin, "int2float",
+			     GST_RANK_SECONDARY, GST_TYPE_INT2FLOAT) ||
+      !gst_element_register (plugin, "float2int",
+			     GST_RANK_SECONDARY, GST_TYPE_FLOAT2INT))
+    return FALSE;
 
   return TRUE;
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "intfloatconvert",
-  plugin_init
-};
+  "Float/integer audio converter",
+  plugin_init,
+  VERSION,
+  GST_LICENSE,
+  GST_PACKAGE,
+  GST_ORIGIN
+)
