@@ -27,21 +27,9 @@
 /* elementfactory information */
 static GstElementDetails mp3parse_details = {
   "MPEG1 Audio Parser",
-  "Codec/Parser",
-  "LGPL",
+  "Codec/Parser/Audio",
   "Parses and frames mpeg1 audio streams (levels 1-3), provides seek",
-  VERSION,
-  "Erik Walthinsen <omega@cse.ogi.edu>",
-  "(C) 1999",
-};
-
-static GstCaps * mp3_type_find (GstByteStream *bs, gpointer data);
-
-static GstTypeDefinition mp3type_definition = {
-  "mp3_audio/mpeg",
-  "audio/mpeg",
-  ".mp3 .mp2 .mp1 .mpga",
-  mp3_type_find,
+  "Erik Walthinsen <omega@cse.ogi.edu>"
 };
 
 static GstPadTemplate*
@@ -95,6 +83,7 @@ enum {
 static GstPadTemplate *sink_temp, *src_temp;
 
 static void	gst_mp3parse_class_init		(GstMPEGAudioParseClass *klass);
+static void	gst_mp3parse_base_init		(GstMPEGAudioParseClass *klass);
 static void	gst_mp3parse_init		(GstMPEGAudioParse *mp3parse);
 
 static void	gst_mp3parse_chain		(GstPad *pad,GstData *_data);
@@ -115,7 +104,8 @@ gst_mp3parse_get_type(void) {
 
   if (!mp3parse_type) {
     static const GTypeInfo mp3parse_info = {
-      sizeof(GstMPEGAudioParseClass),      NULL,
+      sizeof(GstMPEGAudioParseClass),
+      (GBaseInitFunc)gst_mp3parse_base_init,
       NULL,
       (GClassInitFunc)gst_mp3parse_class_init,
       NULL,
@@ -267,76 +257,14 @@ mp3_caps_create (guint layer, guint channels,
   return new;
 }
 
-static GstCaps *
-mp3_type_find (GstByteStream *bs, gpointer private)
+static void
+gst_mp3parse_base_init (GstMPEGAudioParseClass *klass)
 {
-  GstBuffer *buf = NULL;
-  GstCaps *new = NULL;
-  guint8 *data;
-  guint size;
-  guint32 head;
-  guint layer = 0, bitrate = 0, samplerate = 0, channels = 0;
+  GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  /* note that even if we don't get the requested size,
-   * it might still be a (very small) mp3 */
-  gst_bytestream_peek (bs, &buf, GST_MP3_TYPEFIND_MIN_DATA);
-  if (!buf) {
-    goto done;
-  }
-  
-  data = GST_BUFFER_DATA (buf);
-  size = GST_BUFFER_SIZE (buf);
-  
-  while (size >= 4) {
-    head = GUINT32_FROM_BE(*((guint32 *)data));
-    if ((head & 0xffe00000) == 0xffe00000) {
-      guint length;
-      guint prev_layer = 0, prev_bitrate = 0,
-	    prev_channels = 0, prev_samplerate = 0;
-      guint found = 0; /* number of valid headers found */
-      guint pos = 0;
-
-      do {
-        if (!(length = mp3_type_frame_length_from_header (head, &layer,
-							  &channels, &bitrate,
-							  &samplerate))) {
-          break;
-	}
-	if ((prev_layer && prev_layer != layer) || !layer ||
-	    (prev_bitrate && prev_bitrate != bitrate) || !bitrate ||
-	    (prev_samplerate && prev_samplerate != samplerate) || !samplerate ||
-	    (prev_channels && prev_channels != channels) || !channels) {
-          /* this means an invalid property, or a change, which likely
-	   * indicates that this is not a mp3 but just a random bytestream */
-	  break;
-	}
-	prev_layer = layer;
-	prev_bitrate = bitrate;
-	prev_channels = channels;
-	prev_samplerate = samplerate;
-	pos += length;
-	if (++found >= GST_MP3_TYPEFIND_MIN_HEADERS) {
-          /* we're pretty sure that this is mp3 now */
-          new = mp3_caps_create (layer, channels, bitrate, samplerate);
-	  goto done;
-        }
-
-        /* and now, find a new head */
-        head = GUINT32_FROM_BE(*((guint32 *) &(data[pos])));  
-        if ((head & 0xffe00000) != 0xffe00000)
-	  break;
-      } while (TRUE);
-    }
-    data++;
-    size--;
-  }
-
-done:
-  if (buf != NULL) {
-    gst_buffer_unref (buf);
-  }
-
-  return new;
+  gst_element_class_add_pad_template (element_class, sink_temp);
+  gst_element_class_add_pad_template (element_class, src_temp);
+  gst_element_class_set_details (element_class, &mp3parse_details);
 }
 
 static void
@@ -655,35 +583,23 @@ gst_mp3parse_change_state (GstElement *element)
 }
 
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-  GstTypeFactory *type;
-
-  /* create an elementfactory for the mp3parse element */
-  factory = gst_element_factory_new ("mp3parse",
-		                    GST_TYPE_MP3PARSE,
-                                    &mp3parse_details);
-  g_return_val_if_fail (factory != NULL, FALSE);
-
   sink_temp = mp3_sink_factory ();
-  gst_element_factory_add_pad_template (factory, sink_temp);
-
   src_temp = mp3_src_factory ();
-  gst_element_factory_add_pad_template (factory, src_temp);
 
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
-  /* type finding */
-  type = gst_type_factory_new (&mp3type_definition);
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (type));
-
-  return TRUE;
+  return gst_element_register (plugin, "mp3parse",
+			       GST_RANK_NONE, GST_TYPE_MP3PARSE);
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
-  "mp3parse",
-  plugin_init
-};
+  "mpegaudioparse",
+  "MPEG-1 layer 1/2/3 audio parser",
+  plugin_init,
+  VERSION,
+  "LGPL",
+  GST_PACKAGE,
+  GST_ORIGIN
+)
