@@ -18,6 +18,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <string.h>
 #include <gst/gst.h>
 #include <gst/audio/audio.h>
@@ -29,13 +32,10 @@
 /* elementfactory information */
 static GstElementDetails passthrough_details = {
   "Passthrough",
-  "Filter/Audio/Effect",
-  "LGPL",
+  "Filter/Effect/Audio",
   "Transparent filter for audio/raw (boilerplate for effects)",
-  VERSION,
   "Thomas <thomas@apestaart.org>, "\
-  "Andy Wingo <apwingo@eos.ncsu.edu>",
-  "(C) 2001",
+  "Andy Wingo <apwingo@eos.ncsu.edu>"
 };
 
 enum {
@@ -50,21 +50,21 @@ enum {
   ARG_SILENT
 };
 
-static GstPadTemplate*                    
-passthrough_sink_factory (void)            
-{                                         
-  static GstPadTemplate *template = NULL; 
-                                          
-  if (! template) {                        
-    template = gst_pad_template_new 
+static GstPadTemplate*
+passthrough_sink_factory (void)
+{
+  static GstPadTemplate *template = NULL;
+
+  if (! template) {
+    template = gst_pad_template_new
       ("sink", GST_PAD_SINK, GST_PAD_ALWAYS,
-      gst_caps_append (gst_caps_new ("sink_int",  "audio/raw",
+      gst_caps_append (gst_caps_new ("sink_int",  "audio/x-raw-int",
                                      GST_AUDIO_INT_PAD_TEMPLATE_PROPS),
-                       gst_caps_new ("sink_float", "audio/raw",
-                                     GST_AUDIO_FLOAT_MONO_PAD_TEMPLATE_PROPS)),
+                       gst_caps_new ("sink_float", "audio/x-raw-float",
+                                     GST_AUDIO_FLOAT_STANDARD_PAD_TEMPLATE_PROPS)),
       NULL);
-  }                                       
-  return template;                        
+  }
+  return template;
 }
 
 static GstPadTemplate*
@@ -75,9 +75,9 @@ passthrough_src_factory (void)
   if (! template)
     template = gst_pad_template_new 
       ("src", GST_PAD_SRC, GST_PAD_ALWAYS,
-       gst_caps_append (gst_caps_new ("src_float", "audio/raw",
-                                      GST_AUDIO_FLOAT_MONO_PAD_TEMPLATE_PROPS),
-                        gst_caps_new ("src_int", "audio/raw",
+       gst_caps_append (gst_caps_new ("src_float", "audio/x-raw-float",
+                                      GST_AUDIO_FLOAT_STANDARD_PAD_TEMPLATE_PROPS),
+                        gst_caps_new ("src_int", "audio/x-raw-float",
                                       GST_AUDIO_INT_PAD_TEMPLATE_PROPS)),
        NULL);
   
@@ -85,6 +85,7 @@ passthrough_src_factory (void)
 }
 
 static void		passthrough_class_init		(GstPassthroughClass *klass);
+static void		passthrough_base_init		(GstPassthroughClass *klass);
 static void		passthrough_init		(GstPassthrough *filter);
 
 static void		passthrough_set_property	(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
@@ -92,7 +93,7 @@ static void		passthrough_get_property	(GObject *object, guint prop_id, GValue *v
 
 static GstPadLinkReturn passthrough_connect_sink	(GstPad *pad, GstCaps *caps);
 
-static void		passthrough_chain		(GstPad *pad, GstBuffer *buf);
+static void		passthrough_chain		(GstPad *pad, GstData *_data);
 static void inline 	passthrough_fast_float_chain 	(gfloat* data, guint numsamples);
 static void inline 	passthrough_fast_16bit_chain 	(gint16* data, guint numsamples);
 static void inline 	passthrough_fast_8bit_chain	(gint8* data, guint numsamples);
@@ -112,7 +113,7 @@ passthrough_get_bufferpool (GstPad *pad)
 static GstPadLinkReturn
 passthrough_connect_sink (GstPad *pad, GstCaps *caps)
 {
-  const gchar *format;
+  const gchar *mimetype;
   GstPassthrough *filter;
   
   g_return_val_if_fail (pad  != NULL, GST_PAD_LINK_DELAYED);
@@ -122,36 +123,30 @@ passthrough_connect_sink (GstPad *pad, GstCaps *caps)
   g_return_val_if_fail (filter != NULL, GST_PAD_LINK_REFUSED);
   g_return_val_if_fail (GST_IS_PASSTHROUGH (filter), GST_PAD_LINK_REFUSED);
 
-  gst_caps_get_string(caps, "format", &format);
-  
+  mimetype = gst_caps_get_mime(caps);
+
   gst_caps_get_int (caps, "rate", &filter->rate);
   gst_caps_get_int (caps, "channels", &filter->channels);
-  
-  if (strcmp (format, "int") == 0) {
-    filter->format        = GST_PASSTHROUGH_FORMAT_INT;
-    gst_caps_get_int	 (caps, "width",      &filter->width);
+  gst_caps_get_int (caps, "width", &filter->width);
+  gst_caps_get_int (caps, "endianness", &filter->endianness);
+
+  if (strcmp (mimetype, "audio/x-raw-int") == 0) {
+    filter->format = GST_PASSTHROUGH_FORMAT_INT;
+
     gst_caps_get_int	 (caps, "depth",      &filter->depth);
-    gst_caps_get_int	 (caps, "law",        &filter->law);
-    gst_caps_get_int	 (caps, "endianness", &filter->endianness);
     gst_caps_get_boolean (caps, "signed",     &filter->is_signed);
 
     if (! filter->silent) {
-      g_print ("Passthrough : channels %d, rate %d\n",  
-               filter->channels, filter->rate);
+      g_print ("Passthrough : channels %d, rate %d\n", filter->channels, filter->rate);
       g_print ("Passthrough : format int, bit width %d, endianness %d, signed %s\n",
                filter->width, filter->endianness, filter->is_signed ? "yes" : "no");
     }
-  } else if (strcmp (format, "float") == 0) {
-    filter->format     = GST_PASSTHROUGH_FORMAT_FLOAT;
-    gst_caps_get_string (caps, "layout",    &filter->layout);
-    gst_caps_get_float  (caps, "intercept", &filter->intercept);
-    gst_caps_get_float  (caps, "slope",     &filter->slope);
+  } else if (strcmp (mimetype, "audio/x-raw-float") == 0) {
+    filter->format = GST_PASSTHROUGH_FORMAT_FLOAT;
 
     if (! filter->silent) {
-      g_print ("Passthrough : channels %d, rate %d\n",  
-               filter->channels, filter->rate);
-      g_print ("Passthrough : format float, layout %s, intercept %f, slope %f\n",
-               filter->layout, filter->intercept, filter->slope);
+      g_print ("Passthrough : channels %d, rate %d\n", filter->channels, filter->rate);
+      g_print ("Passthrough : format float, width %d\n", filter->width);
     }
   }
 
@@ -167,7 +162,8 @@ gst_passthrough_get_type (void)
 
   if (!passthrough_type) {
     static const GTypeInfo passthrough_info = {
-      sizeof (GstPassthroughClass),      NULL,
+      sizeof (GstPassthroughClass),
+      (GBaseInitFunc) passthrough_base_init,
       NULL,
       (GClassInitFunc) passthrough_class_init,
       NULL,
@@ -181,6 +177,16 @@ gst_passthrough_get_type (void)
   return passthrough_type;
 }
 
+static void
+passthrough_base_init (GstPassthroughClass *klass)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
+
+  gst_element_class_add_pad_template (element_class, passthrough_src_factory ());
+  gst_element_class_add_pad_template (element_class, passthrough_sink_factory ());
+  gst_element_class_set_details (element_class, &passthrough_details);
+}
+
 static void
 passthrough_class_init (GstPassthroughClass *klass)
 {
@@ -216,8 +222,9 @@ passthrough_init (GstPassthrough *filter)
 }
 
 static void
-passthrough_chain (GstPad *pad, GstBuffer *buf)
+passthrough_chain (GstPad *pad, GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstPassthrough *filter;
   gint16 *int_data;
   gfloat *float_data;
@@ -257,7 +264,7 @@ passthrough_chain (GstPad *pad, GstBuffer *buf)
     break;
   }
   
-  gst_pad_push (filter->srcpad, buf);
+  gst_pad_push (filter->srcpad, GST_DATA (buf));
 }
 
 static void inline
@@ -309,24 +316,20 @@ passthrough_get_property (GObject *object, guint prop_id, GValue *value, GParamS
 }
 
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-
-  factory = gst_element_factory_new ("passthrough", GST_TYPE_PASSTHROUGH, &passthrough_details);
-  g_return_val_if_fail (factory != NULL, FALSE);
-  
-  gst_element_factory_add_pad_template (factory, passthrough_src_factory ());
-  gst_element_factory_add_pad_template (factory, passthrough_sink_factory ());
-
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
-  return TRUE;
+  return gst_element_register (plugin, "passthrough",
+			       GST_RANK_NONE, GST_TYPE_PASSTHROUGH);
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "passthrough",
-  plugin_init
-};
+  "Transparent filter for audio/raw (boilerplate for effects)",
+  plugin_init,
+  VERSION,
+  "LGPL",
+  GST_PACKAGE,
+  GST_ORIGIN
+)
