@@ -26,6 +26,8 @@
 #include <string.h>
 #include <zlib.h>
 
+#define g_print(x...)
+
 #define QTDEMUX_GUINT32_GET(a) GUINT32_FROM_BE(*(guint32 *)(a))
 #define QTDEMUX_GUINT16_GET(a) GUINT16_FROM_BE(*(guint16 *)(a))
 #define QTDEMUX_GUINT8_GET(a) (*(guint8 *)(a))
@@ -117,6 +119,11 @@ GST_PAD_TEMPLATE_FACTORY (sink_templ,
     "qtdemux_sink",
     "video/quicktime",
       NULL
+  ),
+  GST_CAPS_NEW (
+    "qtdemux_sink",
+    "audio/x-m4a",
+    NULL
   )
 )
 
@@ -222,7 +229,7 @@ plugin_init (GstPlugin *plugin)
 					videocaps, NULL);
 
   return gst_element_register (plugin, "qtdemux",
-			       GST_RANK_NONE, GST_TYPE_QTDEMUX);
+			       GST_RANK_PRIMARY, GST_TYPE_QTDEMUX);
 }
 
 GST_PLUGIN_DEFINE (
@@ -233,7 +240,6 @@ GST_PLUGIN_DEFINE (
   plugin_init,
   VERSION,
   "LGPL",
-  GST_COPYRIGHT,
   GST_PACKAGE,
   GST_ORIGIN
 )
@@ -373,7 +379,7 @@ static void gst_qtdemux_loop_header (GstElement *element)
 	}while(1);
 
         qtdemux_parse_moov(qtdemux, GST_BUFFER_DATA(moov), length);
-        if(0)qtdemux_node_dump(qtdemux, qtdemux->moov_node);
+        if(1)qtdemux_node_dump(qtdemux, qtdemux->moov_node);
         qtdemux_parse_tree(qtdemux);
         qtdemux->state = QTDEMUX_STATE_MOVIE;
         break;
@@ -1707,6 +1713,7 @@ static GstCaps *qtdemux_audio_caps(GstQTDemux *qtdemux, guint32 fourcc)
     case GST_MAKE_FOURCC('.','m','p','3'):
       /* MPEG layer 3, CBR & VBR (QT4.1 and later) */
       return GST_CAPS_NEW("_mp3_caps","audio/mpeg",
+	  "mpegversion", GST_PROPS_INT(1),
 	  "layer", GST_PROPS_INT(3),
 	  "rate",GST_PROPS_INT_RANGE(1,G_MAXINT),
 	  "channels",GST_PROPS_INT_RANGE(1,G_MAXINT),
