@@ -31,15 +31,12 @@
 
 
 /* elementfactory information */
-static GstElementDetails videotestsrc_details = {
+static GstElementDetails videotestsrc_details = GST_ELEMENT_DETAILS (
   "Video test source",
   "Source/Video",
-  "LGPL",
   "Creates a test video stream",
-  VERSION,
-  "David A. Schleef <ds@schleef.org>",
-  "(C) 2002",
-};
+  "David A. Schleef <ds@schleef.org>"
+);
 
 /* GstVideotestsrc signals and args */
 enum
@@ -60,6 +57,7 @@ enum
   /* FILL ME */
 };
 
+static void gst_videotestsrc_base_init (gpointer g_class);
 static void gst_videotestsrc_class_init (GstVideotestsrcClass * klass);
 static void gst_videotestsrc_init (GstVideotestsrc * videotestsrc);
 static GstElementStateReturn gst_videotestsrc_change_state (GstElement * element);
@@ -73,10 +71,17 @@ static void gst_videotestsrc_get_property (GObject * object, guint prop_id, GVal
 
 static GstData *gst_videotestsrc_get (GstPad * pad);
 
+static const GstQueryType *
+		gst_videotestsrc_get_query_types (GstPad      *pad);
+static gboolean gst_videotestsrc_src_query (GstPad      *pad,
+					    GstQueryType type,
+					    GstFormat   *format,
+					    gint64      *value);
+
 static GstElementClass *parent_class = NULL;
 
 static GstCaps * gst_videotestsrc_get_capslist (void);
-static GstCaps * gst_videotestsrc_get_capslist_size (int height, int width, float rate);
+static GstCaps * gst_videotestsrc_get_capslist_size (int width, int height, float rate);
 
 
 static GstPadTemplate *
@@ -101,7 +106,8 @@ gst_videotestsrc_get_type (void)
 
   if (!videotestsrc_type) {
     static const GTypeInfo videotestsrc_info = {
-      sizeof (GstVideotestsrcClass), NULL,
+      sizeof (GstVideotestsrcClass),
+      gst_videotestsrc_base_init,
       NULL,
       (GClassInitFunc) gst_videotestsrc_class_init,
       NULL,
@@ -136,6 +142,16 @@ gst_videotestsrc_pattern_get_type (void)
   return videotestsrc_pattern_type;
 }
 
+static void
+gst_videotestsrc_base_init (gpointer g_class)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+
+  gst_element_class_set_details (element_class, &videotestsrc_details);
+
+  gst_element_class_add_pad_template (element_class,
+	  GST_PAD_TEMPLATE_GET (videotestsrc_src_template_factory));
+}
 static void
 gst_videotestsrc_class_init (GstVideotestsrcClass * klass)
 {
@@ -311,7 +327,7 @@ gst_videotestsrc_get_capslist (void)
 }
 
 static GstCaps *
-gst_videotestsrc_get_capslist_size (int height, int width, float rate)
+gst_videotestsrc_get_capslist_size (int width, int height, float rate)
 {
   GstCaps *capslist;
   GstCaps *caps;
@@ -394,6 +410,9 @@ gst_videotestsrc_init (GstVideotestsrc * videotestsrc)
   gst_pad_set_get_function (videotestsrc->srcpad, gst_videotestsrc_get);
   gst_pad_set_link_function (videotestsrc->srcpad, gst_videotestsrc_src_link);
   gst_pad_set_unlink_function (videotestsrc->srcpad, gst_videotestsrc_src_unlink);
+  gst_pad_set_query_function (videotestsrc->srcpad, gst_videotestsrc_src_query);
+  gst_pad_set_query_type_function (videotestsrc->srcpad,
+				   gst_videotestsrc_get_query_types);
 
   videotestsrc->pool = NULL;
   gst_videotestsrc_set_pattern(videotestsrc, GST_VIDEOTESTSRC_SMPTE);
@@ -405,6 +424,49 @@ gst_videotestsrc_init (GstVideotestsrc * videotestsrc)
 }
 
 
+static const GstQueryType *
+gst_videotestsrc_get_query_types (GstPad *pad)
+{
+  static const GstQueryType query_types[] = {
+    GST_QUERY_POSITION,
+    0,
+  };
+
+  return query_types;
+} 
+
+static gboolean
+gst_videotestsrc_src_query (GstPad      *pad,
+			    GstQueryType type,
+			    GstFormat   *format,
+			    gint64      *value)
+{
+  gboolean res = FALSE;
+  GstVideotestsrc *videotestsrc = GST_VIDEOTESTSRC (gst_pad_get_parent (pad));
+	        
+  switch (type) {
+    case GST_QUERY_POSITION:
+      switch (*format) {
+        case GST_FORMAT_TIME:
+          *value = videotestsrc->n_frames * GST_SECOND / (double) videotestsrc->rate;
+          res = TRUE;
+          break;
+        case GST_FORMAT_DEFAULT: /* frames */
+          *value = videotestsrc->n_frames;
+          res = TRUE;
+          break;
+        default:
+          break;
+      }
+      break;
+    default:
+      break;
+  }
+
+  return res;
+}
+
+
 static GstData *
 gst_videotestsrc_get (GstPad * pad)
 {
@@ -420,7 +482,11 @@ gst_videotestsrc_get (GstPad * pad)
 
   videotestsrc = GST_VIDEOTESTSRC (gst_pad_get_parent (pad));
 
-  g_return_val_if_fail (videotestsrc->fourcc != NULL, NULL);
+  if (videotestsrc->fourcc != NULL) {
+    gst_element_error (GST_ELEMENT (videotestsrc),
+		       "No color format set - aborting");
+    return NULL;
+  }
 
   newsize = (videotestsrc->width * videotestsrc->height * videotestsrc->bpp) >> 3;
   g_return_val_if_fail (newsize > 0, NULL);
@@ -465,6 +531,7 @@ gst_videotestsrc_get (GstPad * pad)
       (videotestsrc->n_frames * GST_SECOND)/(double)videotestsrc->rate;
     videotestsrc->n_frames++;
   }
+  GST_BUFFER_DURATION (buf) = GST_SECOND / (double) videotestsrc->rate;
 
   return GST_DATA (buf);
 }
@@ -568,28 +635,19 @@ gst_videotestsrc_get_property (GObject * object, guint prop_id, GValue * value,
 
 
 static gboolean
-plugin_init (GModule * module, GstPlugin * plugin)
+plugin_init (GstPlugin * plugin)
 {
-  GstElementFactory *factory;
-
-  /* create an elementfactory for the videotestsrc element */
-  factory = gst_element_factory_new ("videotestsrc", GST_TYPE_VIDEOTESTSRC, &videotestsrc_details);
-  g_return_val_if_fail (factory != NULL, FALSE);
-
-  gst_element_factory_add_pad_template (factory,
-					GST_PAD_TEMPLATE_GET (videotestsrc_src_template_factory));
-
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
-  return TRUE;
+  return gst_element_register (plugin, "videotestsrc", GST_RANK_NONE, GST_TYPE_VIDEOTESTSRC);
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "videotestsrc",
-  plugin_init
-};
-
-
-
+  "Creates a test video stream",
+  plugin_init,
+  VERSION,
+  GST_LICENSE,
+  GST_PACKAGE,
+  GST_ORIGIN
+)
