@@ -33,6 +33,8 @@
 #include <errno.h>
 #include <string.h>
 
+#include <gst/propertyprobe/propertyprobe.h>
+
 #include "gstosselement.h"
 #include "gstossmixer.h"
 
@@ -44,17 +46,17 @@ enum {
 };
 
 /* elementfactory information */
-static GstElementDetails gst_osselement_details = {
-  "Audio Element (OSS)",
+static GstElementDetails gst_osselement_details = GST_ELEMENT_DETAILS (
+  "Audio Mixer (OSS)",
   "Generic/Audio",
-  "LGPL",
-  "Generic OSS element",
-  VERSION,
-  "Erik Walthinsen <omega@cse.ogi.edu>",
-  "(C) 1999",
-};
+  "OSS-based mixer element",
+  "Ronald Bultje <rbultje@ronald.bitfreak.net>"
+);
 
+static void			gst_osselement_base_init	(GstOssElementClass *klass);
 static void 			gst_osselement_class_init	(GstOssElementClass *klass);
+
+static void			gst_ossprobe_interface_init	(GstPropertyProbeInterface *iface);
 static void 			gst_osselement_init		(GstOssElement *oss);
 static void 			gst_osselement_dispose		(GObject *object);
 
@@ -79,7 +81,7 @@ gst_osselement_get_type (void)
   if (!osselement_type) {
     static const GTypeInfo osselement_info = {
       sizeof(GstOssElementClass),
-      NULL,
+      (GBaseInitFunc)gst_osselement_base_init,
       NULL,
       (GClassInitFunc)gst_osselement_class_init,
       NULL,
@@ -98,21 +100,39 @@ gst_osselement_get_type (void)
       NULL,
       NULL,
     };
+    static const GInterfaceInfo ossprobe_info = {
+      (GInterfaceInitFunc) gst_ossprobe_interface_init,
+      NULL,
+      NULL,
+    };
 
     osselement_type = g_type_register_static (GST_TYPE_ELEMENT,
 					      "GstOssElement",
 					      &osselement_info, 0);
     g_type_add_interface_static (osselement_type,
-				 GST_TYPE_INTERFACE,
+				 GST_TYPE_IMPLEMENTS_INTERFACE,
 				 &ossiface_info);
     g_type_add_interface_static (osselement_type,
 				 GST_TYPE_MIXER,
 				 &ossmixer_info);
+    g_type_add_interface_static (osselement_type,
+				 GST_TYPE_PROPERTY_PROBE,
+				 &ossprobe_info);
   }
 
   return osselement_type;
 }
 
+static void
+gst_osselement_base_init (GstOssElementClass *klass)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
+
+  klass->device_combinations = NULL;
+
+  gst_element_class_set_details (element_class, &gst_osselement_details);
+}
+
 static void
 gst_osselement_class_init (GstOssElementClass *klass) 
 {
@@ -142,6 +162,218 @@ gst_osselement_class_init (GstOssElementClass *klass)
   gstelement_class->change_state = gst_osselement_change_state;
 }
 
+static const GList *
+gst_ossprobe_get_properties (GstPropertyProbe *probe)
+{
+  GObjectClass *klass = G_OBJECT_GET_CLASS (probe);
+  static GList *list = NULL;
+
+  if (!list) {
+    list = g_list_append (NULL, g_object_class_find_property (klass, "device"));
+  }
+
+  return list;
+}
+
+static void
+gst_osselement_probe (gchar  *device_base,
+		      gint    device_num,
+		      gchar **put)
+{
+  gchar *device;
+  struct stat s;
+
+  /* only if yet unfilled */
+  if (*put != NULL)
+    return;
+
+  if (device_num == 0)
+    device = g_strdup (device_base);
+  else
+    device = g_strdup_printf ("%s%d", device_base, device_num);
+
+  if (lstat (device, &s) || !S_ISCHR (s.st_mode))
+    goto end;
+
+  *put = device;
+  return;
+
+end:
+  g_free (device);
+}
+
+static gboolean
+gst_osselement_class_probe_devices (GstOssElementClass *klass,
+				    gboolean            check)
+{
+  static gboolean init = FALSE;
+  static GList *device_combinations;
+
+  if (!init && !check) {
+    gchar *dsp_base[] = { "/dev/dsp", "/dev/sound/dsp", NULL };
+    gchar *mixer_base[] = { "/dev/mixer", "/dev/sound/mixer", NULL };
+    GstOssDeviceCombination devices[16];
+    gint n;
+
+    while (device_combinations) {
+      GList *item = device_combinations;
+      GstOssDeviceCombination *combi = item->data;
+
+      device_combinations = g_list_remove (device_combinations, item);
+
+      g_free (combi->dsp);
+      g_free (combi->mixer);
+      g_free (combi);
+    }
+
+    /* probe for all /dev entries */
+    memset (devices, 0, sizeof (devices));
+
+    /* OSS (without devfs) allows at max. 16 devices */
+    for (n = 0; n < 16; n++) {
+      gint base;
+
+      for (base = 0; dsp_base[base] != NULL; base++)
+        gst_osselement_probe (dsp_base[base], n, &devices[n].dsp);
+
+      for (base = 0; mixer_base[base] != NULL; base++)
+        gst_osselement_probe (mixer_base[base], n, &devices[n].mixer);
+    }
+
+    /* does the device exist (can we open them)? */
+    for (n = 0; n < 16; n++) {
+      gint fd;
+
+      if (!devices[n].dsp)
+        continue;
+
+      /* we just check the dsp. we assume the mixer always works.
+       * we don't need a mixer anyway (says OSS)... */
+      if ((fd = open (devices[n].dsp, O_RDONLY)) > 0 || errno == EBUSY) {
+        GstOssDeviceCombination *combi;
+
+        if (fd > 0)
+          close (fd);
+
+        /* yay! \o/ */
+        combi = g_new0 (GstOssDeviceCombination, 1);
+        combi->dsp   = devices[n].dsp;
+        combi->mixer = devices[n].mixer;
+        devices[n].dsp = devices[n].mixer = NULL;
+
+        device_combinations = g_list_append (device_combinations, combi);
+      }
+    }
+
+    /* free */
+    for (n = 0; n < 16; n++) {
+      if (devices[n].dsp)
+        g_free (devices[n].dsp);
+
+      if (devices[n].mixer)
+        g_free (devices[n].mixer);
+    }
+
+    init = TRUE;
+  }
+
+  klass->device_combinations = device_combinations;
+
+  return init;
+}
+
+static GValueArray *
+gst_osselement_class_list_devices (GstOssElementClass *klass)
+{
+  GValueArray *array;
+  GValue value = { 0 };
+  GList *item;
+
+  if (!klass->device_combinations)
+    return NULL;
+
+  array = g_value_array_new (g_list_length (klass->device_combinations));
+  item = klass->device_combinations;
+  g_value_init (&value, G_TYPE_STRING);
+  while (item) {
+    GstOssDeviceCombination *combi = item->data;
+
+    g_value_set_string (&value, combi->dsp);
+    g_value_array_append (array, &value);
+
+    item = item->next;
+  }
+  g_value_unset (&value);
+
+  return array;
+}
+
+static void
+gst_ossprobe_probe_property (GstPropertyProbe *probe,
+			     guint             prop_id,
+			     const GParamSpec *pspec)
+{
+  GstOssElementClass *klass = GST_OSSELEMENT_GET_CLASS (probe);
+
+  switch (prop_id) {
+    case ARG_DEVICE:
+      gst_osselement_class_probe_devices (klass, FALSE);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (probe, prop_id, pspec);
+      break;
+  }
+}
+
+static gboolean
+gst_ossprobe_needs_probe (GstPropertyProbe *probe,
+			  guint             prop_id,
+			  const GParamSpec *pspec)
+{
+  GstOssElementClass *klass = GST_OSSELEMENT_GET_CLASS (probe);
+  gboolean ret = FALSE;
+
+  switch (prop_id) {
+    case ARG_DEVICE:
+      ret = !gst_osselement_class_probe_devices (klass, TRUE);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (probe, prop_id, pspec);
+      break;
+  }
+
+  return ret;
+}
+
+static GValueArray *
+gst_ossprobe_get_values (GstPropertyProbe *probe,
+			 guint             prop_id,
+			 const GParamSpec *pspec)
+{
+  GstOssElementClass *klass = GST_OSSELEMENT_GET_CLASS (probe);
+  GValueArray *array = NULL;
+
+  switch (prop_id) {
+    case ARG_DEVICE:
+      array = gst_osselement_class_list_devices (klass);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (probe, prop_id, pspec);
+      break;
+  }
+
+  return array;
+}
+
+static void
+gst_ossprobe_interface_init (GstPropertyProbeInterface *iface)
+{
+  iface->get_properties = gst_ossprobe_get_properties;
+  iface->probe_property = gst_ossprobe_probe_property;
+  iface->needs_probe    = gst_ossprobe_needs_probe;
+  iface->get_values     = gst_ossprobe_get_values;
+}
+
 static void 
 gst_osselement_init (GstOssElement *oss) 
 {
@@ -412,6 +644,8 @@ gst_osselement_open_audio (GstOssElement *oss)
     if (GST_PAD_IS_SINK (firstpad)) {
       mode = GST_OSSELEMENT_WRITE;
     }
+  } else {
+    goto do_mixer;
   }
 
   /* first try to open the sound card */
@@ -515,6 +749,7 @@ gst_osselement_open_audio (GstOssElement *oss)
 
   oss->caps = caps;
 
+do_mixer:
   gst_ossmixer_build_list (oss);
 
   return TRUE;
@@ -523,10 +758,11 @@ gst_osselement_open_audio (GstOssElement *oss)
 static void
 gst_osselement_close_audio (GstOssElement *oss)
 {
+  gst_ossmixer_free_list (oss);
+
   if (oss->fd < 0) 
     return;
 
-  gst_ossmixer_free_list (oss);
   close(oss->fd);
   oss->fd = -1;
 }
@@ -604,15 +840,33 @@ gst_osselement_set_property (GObject *object,
     case ARG_DEVICE:
       /* disallow changing the device while it is opened
          get_property("device") should return the right one */
-      if (gst_element_get_state (GST_ELEMENT (oss)) != GST_STATE_NULL) {
+      if (gst_element_get_state (GST_ELEMENT (oss)) == GST_STATE_NULL) {
         g_free (oss->device);
         oss->device = g_strdup (g_value_get_string (value));
+
+        /* let's assume that if we have a device map for the mixer,
+	 * we're allowed to do all that automagically here */
+        if (GST_OSSELEMENT_GET_CLASS (oss)->device_combinations != NULL) {
+          GList *list = GST_OSSELEMENT_GET_CLASS (oss)->device_combinations;
+
+          while (list) {
+            GstOssDeviceCombination *combi = list->data;
+
+            if (!strcmp (combi->dsp, oss->device)) {
+              g_free (oss->mixer_dev);
+              oss->mixer_dev = g_strdup (combi->mixer);
+              break;
+            }
+
+            list = list->next;
+          }
+        }
       }
       break;
     case ARG_MIXERDEV:
       /* disallow changing the device while it is opened
          get_property("mixerdev") should return the right one */
-      if (gst_element_get_state (GST_ELEMENT (oss)) != GST_STATE_NULL) {
+      if (gst_element_get_state (GST_ELEMENT (oss)) == GST_STATE_NULL) {
         g_free (oss->mixer_dev);
         oss->mixer_dev = g_strdup (g_value_get_string (value));
       }
@@ -673,17 +927,3 @@ gst_osselement_change_state (GstElement *element)
   return GST_STATE_SUCCESS;
 }
 
-gboolean 
-gst_osselement_factory_init (GstPlugin *plugin) 
-{ 
-  GstElementFactory *factory;
-
-  factory = gst_element_factory_new ("osselement",
-				     GST_TYPE_OSSELEMENT,
-				     &gst_osselement_details);
-  g_return_val_if_fail (factory != NULL, FALSE);
-
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
-  return TRUE;
-}
