@@ -21,22 +21,26 @@
 #include <config.h>
 #endif
 
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <errno.h>
+#include <unistd.h>
+
 #include <string.h>
 #include "v4l_calls.h"
 #include "gstv4ltuner.h"
 #include "gstv4lxoverlay.h"
 #include "gstv4lcolorbalance.h"
 
+#include <gst/propertyprobe/propertyprobe.h>
+
 /* elementfactory information */
-static GstElementDetails gst_v4lelement_details = {
+static GstElementDetails gst_v4lelement_details = GST_ELEMENT_DETAILS (
   "Generic video4linux Element",
   "Generic/Video",
-  "LGPL",
   "Generic plugin for handling common video4linux calls",
-  VERSION,
-  "Ronald Bultje <rbultje@ronald.bitfreak.net>",
-  "(C) 2001",
-};
+  "Ronald Bultje <rbultje@ronald.bitfreak.net>"
+);
 
 /* V4lElement signals and args */
 enum {
@@ -54,6 +58,7 @@ enum {
 };
 
 
+static void gst_v4lelement_base_init	(gpointer g_class);
 static void gst_v4lelement_class_init   (GstV4lElementClass *klass);
 static void gst_v4lelement_init         (GstV4lElement *v4lelement);
 static void gst_v4lelement_dispose      (GObject       *object);
@@ -73,8 +78,8 @@ static GstElementClass *parent_class = NULL;
 static guint gst_v4lelement_signals[LAST_SIGNAL] = { 0 };
 
 static gboolean
-gst_v4l_iface_supported (GstInterface *iface,
-			 GType         iface_type)
+gst_v4l_iface_supported (GstImplementsInterface *iface,
+			 GType                   iface_type)
 {
   GstV4lElement *v4lelement = GST_V4LELEMENT (iface);
 
@@ -93,12 +98,165 @@ gst_v4l_iface_supported (GstInterface *iface,
 }
 
 static void
-gst_v4l_interface_init (GstInterfaceClass *klass)
+gst_v4l_interface_init (GstImplementsInterfaceClass *klass)
 {
   /* default virtual functions */
   klass->supported = gst_v4l_iface_supported;
 }
 
+static const GList *
+gst_v4l_probe_get_properties (GstPropertyProbe *probe)
+{
+  GObjectClass *klass = G_OBJECT_GET_CLASS (probe);
+  static GList *list = NULL;
+
+  if (!list) {
+    list = g_list_append (NULL, g_object_class_find_property (klass, "device"));
+  }
+
+  return list;
+}
+
+static gboolean
+gst_v4l_class_probe_devices (GstV4lElementClass *klass,
+			     gboolean            check)
+{
+  static gboolean init = FALSE;
+  static GList *devices = NULL;
+
+  if (!init && !check) {
+    gchar *dev_base[] = { "/dev/video", "/dev/v4l/video", NULL };
+    gint base, n, fd;
+
+    while (devices) {
+      GList *item = devices;
+      gchar *device = item->data;
+
+      devices = g_list_remove (devices, item);
+      g_free (device);
+    }
+
+    /* detect /dev entries */
+    for (n = 0; n < 64; n++) {
+      for (base = 0; dev_base[base] != NULL; base++) {
+        struct stat s;
+        gchar *device = g_strdup_printf ("%s%d", dev_base[base], n);
+
+        /* does the /dev/ entry exist at all? */
+        if (stat (device, &s) == 0) {
+          /* yes: is a device attached? */
+          if ((fd = open (device, O_RDONLY)) > 0 || errno == EBUSY) {
+            if (fd > 0)
+              close (fd);
+
+            devices = g_list_append (devices, device);
+            break;
+          }
+        }
+        g_free (device);
+      }
+    }
+
+    init = TRUE;
+  }
+
+  klass->devices = devices;
+
+  return init;
+}
+
+static void
+gst_v4l_probe_probe_property (GstPropertyProbe *probe,
+			      guint             prop_id,
+			      const GParamSpec *pspec)
+{
+  GstV4lElementClass *klass = GST_V4LELEMENT_GET_CLASS (probe);
+
+  switch (prop_id) {
+    case ARG_DEVICE:
+      gst_v4l_class_probe_devices (klass, FALSE);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (probe, prop_id, pspec);
+      break;
+  }
+}
+
+static gboolean
+gst_v4l_probe_needs_probe (GstPropertyProbe *probe,
+			   guint             prop_id,
+			   const GParamSpec *pspec)
+{
+  GstV4lElementClass *klass = GST_V4LELEMENT_GET_CLASS (probe);
+  gboolean ret = FALSE;
+
+  switch (prop_id) {
+    case ARG_DEVICE:
+      ret = !gst_v4l_class_probe_devices (klass, TRUE);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (probe, prop_id, pspec);
+      break;
+  }
+
+  return ret;
+}
+
+static GValueArray *
+gst_v4l_class_list_devices (GstV4lElementClass *klass)
+{
+  GValueArray *array;
+  GValue value = { 0 };
+  GList *item;
+
+  if (!klass->devices)
+    return NULL;
+
+  array = g_value_array_new (g_list_length (klass->devices));
+  item = klass->devices;
+  g_value_init (&value, G_TYPE_STRING);
+  while (item) {
+    gchar *device = item->data;
+
+    g_value_set_string (&value, device);
+    g_value_array_append (array, &value);
+
+    item = item->next;
+  }
+  g_value_unset (&value);
+
+  return array;
+}
+
+static GValueArray *
+gst_v4l_probe_get_values (GstPropertyProbe *probe,
+			  guint             prop_id,
+			  const GParamSpec *pspec)
+{
+  GstV4lElementClass *klass = GST_V4LELEMENT_GET_CLASS (probe);
+  GValueArray *array = NULL;
+
+  switch (prop_id) {
+    case ARG_DEVICE:
+      array = gst_v4l_class_list_devices (klass);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (probe, prop_id, pspec);
+      break;
+  }
+
+  return array;
+}
+
+static void
+gst_v4l_property_probe_interface_init (GstPropertyProbeInterface *iface)
+{
+  iface->get_properties = gst_v4l_probe_get_properties;
+  iface->probe_property = gst_v4l_probe_probe_property;
+  iface->needs_probe    = gst_v4l_probe_needs_probe;
+  iface->get_values     = gst_v4l_probe_get_values;
+}
+
 #define GST_TYPE_V4L_DEVICE_FLAGS (gst_v4l_device_get_type ())
 GType
 gst_v4l_device_get_type (void)
@@ -134,7 +292,7 @@ gst_v4lelement_get_type (void)
   if (!v4lelement_type) {
     static const GTypeInfo v4lelement_info = {
       sizeof(GstV4lElementClass),
-      NULL,
+      gst_v4lelement_base_init,
       NULL,
       (GClassInitFunc)gst_v4lelement_class_init,
       NULL,
@@ -164,13 +322,18 @@ gst_v4lelement_get_type (void)
       NULL,
       NULL,
     };
+    static const GInterfaceInfo v4l_propertyprobe_info = {
+      (GInterfaceInitFunc) gst_v4l_property_probe_interface_init,
+      NULL,
+      NULL,
+    };
 
     v4lelement_type = g_type_register_static(GST_TYPE_ELEMENT,
 					     "GstV4lElement",
 					     &v4lelement_info, 0);
 
     g_type_add_interface_static (v4lelement_type,
-				 GST_TYPE_INTERFACE,
+				 GST_TYPE_IMPLEMENTS_INTERFACE,
 				 &v4liface_info);
     g_type_add_interface_static (v4lelement_type,
 				 GST_TYPE_TUNER,
@@ -181,12 +344,26 @@ gst_v4lelement_get_type (void)
     g_type_add_interface_static (v4lelement_type,
 				 GST_TYPE_COLOR_BALANCE,
 				 &v4l_colorbalance_info);
+    g_type_add_interface_static (v4lelement_type,
+				 GST_TYPE_PROPERTY_PROBE,
+				 &v4l_propertyprobe_info);
   }
 
   return v4lelement_type;
 }
 
 
+static void 
+gst_v4lelement_base_init (gpointer g_class)
+{
+  GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
+  GstV4lElementClass *klass = GST_V4LELEMENT_CLASS (g_class);
+
+  klass->devices = NULL;
+  
+  gst_element_class_set_details (gstelement_class, &gst_v4lelement_details);
+}
+
 static void
 gst_v4lelement_class_init (GstV4lElementClass *klass)
 {
@@ -377,23 +554,3 @@ gst_v4lelement_change_state (GstElement *element)
 
   return GST_STATE_SUCCESS;
 }
-
-
-gboolean
-gst_v4lelement_factory_init (GstPlugin *plugin)
-{
-  GstElementFactory *factory;
-
-  /* actually, we can survive without it, but I'll create
-   * that handling later on. */
-  if (!gst_library_load ("xwindowlistener"))
-    return FALSE;
-
-  /* create an elementfactory for the v4lelement */
-  factory = gst_element_factory_new("v4lelement", GST_TYPE_V4LELEMENT,
-                                   &gst_v4lelement_details);
-  g_return_val_if_fail(factory != NULL, FALSE);
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
-  return TRUE;
-}
