@@ -17,6 +17,10 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -33,7 +37,7 @@
 #endif
 
 #define DEBUG(format, args...) \
-	GST_DEBUG_ELEMENT(GST_CAT_PLUGIN_INFO, \
+	GST_DEBUG_OBJECT (\
 		GST_ELEMENT(v4lmjpegsink), \
 		"V4LMJPEGSINK: " format, ##args)
 
@@ -440,7 +444,7 @@ guint8 *
 gst_v4lmjpegsink_get_buffer (GstV4lMjpegSink *v4lmjpegsink,
                              gint            num)
 {
-  DEBUG("gst_v4lmjpegsink_get_buffer(), num = %d", num);
+  /*DEBUG("gst_v4lmjpegsink_get_buffer(), num = %d", num);*/
 
   if (!GST_V4L_IS_ACTIVE(GST_V4LELEMENT(v4lmjpegsink)) ||
       !GST_V4L_IS_OPEN(GST_V4LELEMENT(v4lmjpegsink)))
