@@ -17,6 +17,10 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -37,12 +41,12 @@
 #endif
 
 #define DEBUG(format, args...) \
-	GST_DEBUG_ELEMENT(GST_CAT_PLUGIN_INFO, \
+	GST_DEBUG_OBJECT (\
 		GST_ELEMENT(v4lsrc), \
 		"V4LSRC: " format, ##args)
 
 /* palette names */
-char *palette_name[] = {
+static const char *palette_name[] = {
   "",                          /* 0 */
   "grayscale",                 /* VIDEO_PALETTE_GREY */
   "Hi-420",                    /* VIDEO_PALETTE_HI420 */
@@ -62,28 +66,34 @@ char *palette_name[] = {
   "YUV-4:1:0 (planar)"         /* VIDEO_PALETTE_YUV410P */
 };
 
+enum {
+  QUEUE_STATE_ERROR = -1,
+  QUEUE_STATE_READY_FOR_QUEUE,
+  QUEUE_STATE_QUEUED,
+  QUEUE_STATE_SYNCED,
+};
 
 /******************************************************
  * gst_v4lsrc_queue_frame():
  *   queue a frame for capturing
+ *   Requires queue_state lock to be held!
  * return value: TRUE on success, FALSE on error
  ******************************************************/
 
 static gboolean
 gst_v4lsrc_queue_frame (GstV4lSrc *v4lsrc,
-                        gint      num)
+                        gint       num)
 {
   DEBUG("queueing frame %d", num);
 
-  v4lsrc->mmap.frame = num;
-
-  if (v4lsrc->frame_queued[num] < 0)
-  {
-    v4lsrc->frame_queued[num] = 0;
-    return TRUE;
+  if (v4lsrc->frame_queue_state[num] != QUEUE_STATE_READY_FOR_QUEUE) {
+    return FALSE;
   }
 
-  if (ioctl(GST_V4LELEMENT(v4lsrc)->video_fd, VIDIOCMCAPTURE, &(v4lsrc->mmap)) < 0)
+  v4lsrc->mmap.frame = num;
+
+  if (ioctl(GST_V4LELEMENT(v4lsrc)->video_fd,
+            VIDIOCMCAPTURE, &(v4lsrc->mmap)) < 0)
   {
     gst_element_error(GST_ELEMENT(v4lsrc),
       "Error queueing a buffer (%d): %s",
@@ -91,131 +101,53 @@ gst_v4lsrc_queue_frame (GstV4lSrc *v4lsrc,
     return FALSE;
   }
 
-  v4lsrc->frame_queued[num] = 1;
-
-  g_mutex_lock(v4lsrc->mutex_queued_frames);
-  v4lsrc->num_queued_frames++;
-  g_cond_broadcast(v4lsrc->cond_queued_frames);
-  g_mutex_unlock(v4lsrc->mutex_queued_frames);
+  v4lsrc->frame_queue_state[num] = QUEUE_STATE_QUEUED;
+  v4lsrc->num_queued++;
 
   return TRUE;
 }
 
-
 /******************************************************
- * gst_v4lsrc_soft_sync_thread()
- *   syncs on frames and signals the main thread
- * purpose: actually get the correct frame timestamps
- ******************************************************/
+ * gst_v4lsrc_hard_sync_frame(GstV4lSrc *v4lsrc,gint num)
+ *   sync a frame and set the timestamp correctly
+ *   Requires queue_state lock to be held
+ *****************************************************/
 
-static void *
-gst_v4lsrc_soft_sync_thread (void *arg)
+static gboolean 
+gst_v4lsrc_sync_frame (GstV4lSrc *v4lsrc, gint num)
 {
-  GstV4lSrc *v4lsrc = GST_V4LSRC(arg);
-  gint frame = 0;
+  DEBUG("Syncing on frame %d",num);
 
-  DEBUG("starting software sync thread");
-
-#if 0
-  /* Allow easy shutting down by other processes... */
-  pthread_setcancelstate( PTHREAD_CANCEL_ENABLE, NULL );
-  pthread_setcanceltype( PTHREAD_CANCEL_ASYNCHRONOUS, NULL );
-#endif
-
-  while (1)
-  {
-    /* are there queued frames left? */
-    g_mutex_lock(v4lsrc->mutex_queued_frames);
-    if (v4lsrc->num_queued_frames < MIN_BUFFERS_QUEUED)
-    {
-      if (v4lsrc->frame_queued[frame] < 0)
-        break;
-
-      DEBUG("Waiting for new frames to be queued (%d < %d)",
-        v4lsrc->num_queued_frames, MIN_BUFFERS_QUEUED);
-
-      g_cond_wait(v4lsrc->cond_queued_frames, v4lsrc->mutex_queued_frames);
-    }
-    g_mutex_unlock(v4lsrc->mutex_queued_frames);
-
-    if (!v4lsrc->num_queued_frames)
-    {
-      DEBUG("Got signal to exit...");
-      goto end;
-    }
-
-    /* sync on the frame */
-    DEBUG("Sync\'ing on frame %d", frame);
-retry:
-    if (ioctl(GST_V4LELEMENT(v4lsrc)->video_fd, VIDIOCSYNC, &frame) < 0)
-    {
-      /* if the sync() got interrupted, we can retry */
-      if (errno == EINTR)
-        goto retry;
-      gst_element_error(GST_ELEMENT(v4lsrc),
-        "Error syncing on a buffer (%d): %s",
-        frame, g_strerror(errno));
-      g_mutex_lock(v4lsrc->mutex_soft_sync);
-      v4lsrc->isready_soft_sync[frame] = -1;
-      g_cond_broadcast(v4lsrc->cond_soft_sync[frame]);
-      g_mutex_unlock(v4lsrc->mutex_soft_sync);
-      goto end;
-    }
-    else
-    {
-      g_mutex_lock(v4lsrc->mutex_soft_sync);
-      gettimeofday(&(v4lsrc->timestamp_soft_sync[frame]), NULL);
-      v4lsrc->isready_soft_sync[frame] = 1;
-      g_cond_broadcast(v4lsrc->cond_soft_sync[frame]);
-      g_mutex_unlock(v4lsrc->mutex_soft_sync);
-    }
-
-    g_mutex_lock(v4lsrc->mutex_queued_frames);
-    v4lsrc->num_queued_frames--;
-    v4lsrc->frame_queued[frame] = 0;
-    g_mutex_unlock(v4lsrc->mutex_queued_frames);
-
-    frame = (frame+1)%v4lsrc->mbuf.frames;
-  }
-
-end:
-  DEBUG("Software sync thread got signalled to exit");
-  g_thread_exit(NULL);
-  return NULL;
-}
-
-
-/******************************************************
- * gst_v4lsrc_sync_frame():
- *   sync on a frame for capturing
- * return value: TRUE on success, FALSE on error
- ******************************************************/
-
-static gboolean
-gst_v4lsrc_sync_next_frame (GstV4lSrc *v4lsrc,
-                            gint      *num)
-{
-  *num = v4lsrc->sync_frame = (v4lsrc->sync_frame + 1)%v4lsrc->mbuf.frames;
-
-  DEBUG("syncing on next frame (%d)", *num);
-
-  /* "software sync()" on the frame */
-  g_mutex_lock(v4lsrc->mutex_soft_sync);
-  if (v4lsrc->isready_soft_sync[*num] == 0)
-  {
-    DEBUG("Waiting for frame %d to be synced on", *num);
-    g_cond_wait(v4lsrc->cond_soft_sync[*num], v4lsrc->mutex_soft_sync);
-  }
-
-  if (v4lsrc->isready_soft_sync[*num] < 0)
+  if (v4lsrc->frame_queue_state[num] != QUEUE_STATE_QUEUED) {
     return FALSE;
-  v4lsrc->isready_soft_sync[*num] = 0;
-  g_mutex_unlock(v4lsrc->mutex_soft_sync);
+  }
+
+  while (ioctl(GST_V4LELEMENT(v4lsrc)->video_fd, VIDIOCSYNC, &num) < 0) {
+    /* if the sync() got interrupted, we can retry */
+    if (errno != EINTR) {
+      v4lsrc->frame_queue_state[num] = QUEUE_STATE_ERROR;
+      gst_element_error(GST_ELEMENT(v4lsrc),
+         "Error syncing on a buffer (%d): %s",
+          num, g_strerror(errno));
+      return FALSE;
+    }
+    DEBUG("Sync got interrupted");
+  }
+
+  if (v4lsrc->clock) {
+    v4lsrc->timestamp_sync = gst_clock_get_time(v4lsrc->clock);
+  } else {
+    GTimeVal time;
+    g_get_current_time(&time);
+    v4lsrc->timestamp_sync = GST_TIMEVAL_TO_TIME(time);
+  }
+
+  v4lsrc->frame_queue_state[num] = QUEUE_STATE_SYNCED;
+  v4lsrc->num_queued--;
 
   return TRUE;
 }
 
-
 /******************************************************
  * gst_v4lsrc_set_capture():
  *   set capture parameters, palette = VIDEO_PALETTE_*
@@ -251,8 +183,6 @@ gst_v4lsrc_set_capture (GstV4lSrc *v4lsrc,
 gboolean
 gst_v4lsrc_capture_init (GstV4lSrc *v4lsrc)
 {
-  int n;
-
   DEBUG("initting capture subsystem");
   GST_V4L_CHECK_OPEN(GST_V4LELEMENT(v4lsrc));
   GST_V4L_CHECK_NOT_ACTIVE(GST_V4LELEMENT(v4lsrc));
@@ -278,52 +208,17 @@ gst_v4lsrc_capture_init (GstV4lSrc *v4lsrc)
     v4lsrc->mbuf.frames, palette_name[v4lsrc->mmap.format],
     v4lsrc->mbuf.size/(v4lsrc->mbuf.frames*1024));
 
-  /* keep trakc of queued buffers */
-  v4lsrc->frame_queued = (gint8 *) malloc(sizeof(gint8) * v4lsrc->mbuf.frames);
-  if (!v4lsrc->frame_queued)
-  {
-    gst_element_error(GST_ELEMENT(v4lsrc),
-      "Error creating buffer tracker: %s",
-      g_strerror(errno));
-    return FALSE;
-  }
-  for (n=0;n<v4lsrc->mbuf.frames;n++)
-    v4lsrc->frame_queued[n] = 0;
+  /* keep track of queued buffers */
+  v4lsrc->frame_queue_state = (gint8 *)
+    g_malloc(sizeof(gint8) * v4lsrc->mbuf.frames);
 
-  /* init the GThread stuff */
-  v4lsrc->mutex_soft_sync = g_mutex_new();
-  v4lsrc->isready_soft_sync = (gint8 *) malloc(sizeof(gint8) * v4lsrc->mbuf.frames);
-  if (!v4lsrc->isready_soft_sync)
-  {
-    gst_element_error(GST_ELEMENT(v4lsrc),
-      "Error creating software-sync buffer tracker: %s",
-      g_strerror(errno));
-    return FALSE;
-  }
-  for (n=0;n<v4lsrc->mbuf.frames;n++)
-    v4lsrc->isready_soft_sync[n] = 0;
-  v4lsrc->timestamp_soft_sync = (struct timeval *)
-    malloc(sizeof(struct timeval) * v4lsrc->mbuf.frames);
-  if (!v4lsrc->timestamp_soft_sync)
-  {
-    gst_element_error(GST_ELEMENT(v4lsrc),
-      "Error creating software-sync timestamp tracker: %s",
-      g_strerror(errno));
-    return FALSE;
-  }
-  v4lsrc->cond_soft_sync = (GCond **) malloc( sizeof(GCond *) * v4lsrc->mbuf.frames);
-  if (!v4lsrc->cond_soft_sync)
-  {
-    gst_element_error(GST_ELEMENT(v4lsrc),
-      "Error creating software-sync condition tracker: %s",
-      g_strerror(errno));
-    return FALSE;
-  }
-  for (n=0;n<v4lsrc->mbuf.frames;n++)
-    v4lsrc->cond_soft_sync[n] = g_cond_new();
+  /* track how often to use each frame */
+  v4lsrc->use_num_times = (gint *)
+    g_malloc(sizeof(gint) * v4lsrc->mbuf.frames);
 
-  v4lsrc->mutex_queued_frames = g_mutex_new();
-  v4lsrc->cond_queued_frames = g_cond_new();
+  /* lock for the frame_state */
+  v4lsrc->mutex_queue_state = g_mutex_new();
+  v4lsrc->cond_queue_state = g_cond_new();
 
   /* Map the buffers */
   GST_V4LELEMENT(v4lsrc)->buffer = mmap(0, v4lsrc->mbuf.size, 
@@ -350,32 +245,31 @@ gst_v4lsrc_capture_init (GstV4lSrc *v4lsrc)
 gboolean
 gst_v4lsrc_capture_start (GstV4lSrc *v4lsrc)
 {
-  GError *error = NULL;
   int n;
 
   DEBUG("starting capture");
   GST_V4L_CHECK_OPEN(GST_V4LELEMENT(v4lsrc));
   GST_V4L_CHECK_ACTIVE(GST_V4LELEMENT(v4lsrc));
 
-  v4lsrc->num_queued_frames = 0;
+  g_mutex_lock(v4lsrc->mutex_queue_state);
 
-  /* queue all buffers, this starts streaming capture */
-  for (n=0;n<v4lsrc->mbuf.frames;n++)
-    if (!gst_v4lsrc_queue_frame(v4lsrc, n))
+  v4lsrc->quit = FALSE;
+  v4lsrc->num_queued = 0;
+  v4lsrc->sync_frame = 0;
+  v4lsrc->queue_frame = 0;
+
+  /* set all buffers ready to queue , this starts streaming capture */
+  for (n=0;n<v4lsrc->mbuf.frames;n++) {
+    v4lsrc->frame_queue_state[n] = QUEUE_STATE_READY_FOR_QUEUE;
+    if (!gst_v4lsrc_queue_frame(v4lsrc, n)) {
+      g_mutex_unlock(v4lsrc->mutex_queue_state);
+      gst_v4lsrc_capture_stop(v4lsrc);
       return FALSE;
-
-  v4lsrc->sync_frame = -1;
-
-  /* start the sync() thread (correct timestamps) */
-  v4lsrc->thread_soft_sync = g_thread_create(gst_v4lsrc_soft_sync_thread,
-    (void *) v4lsrc, TRUE, &error);
-  if (!v4lsrc->thread_soft_sync)
-  {
-    gst_element_error(GST_ELEMENT(v4lsrc),
-      "Failed to create software sync thread: %s",error->message);
-    return FALSE;
+    }
   }
 
+  g_mutex_unlock(v4lsrc->mutex_queue_state);
+
   return TRUE;
 }
 
@@ -393,9 +287,41 @@ gst_v4lsrc_grab_frame (GstV4lSrc *v4lsrc, gint *num)
   GST_V4L_CHECK_OPEN(GST_V4LELEMENT(v4lsrc));
   GST_V4L_CHECK_ACTIVE(GST_V4LELEMENT(v4lsrc));
 
+  g_mutex_lock(v4lsrc->mutex_queue_state);
+
+  /* do we have enough frames? */
+  while (v4lsrc->num_queued < MIN_BUFFERS_QUEUED ||
+         v4lsrc->frame_queue_state[v4lsrc->queue_frame] ==
+           QUEUE_STATE_READY_FOR_QUEUE) {
+    while (v4lsrc->frame_queue_state[v4lsrc->queue_frame] !=
+             QUEUE_STATE_READY_FOR_QUEUE &&
+           !v4lsrc->quit) {
+      GST_DEBUG (
+                "Waiting for frames to become available (%d < %d)",
+                v4lsrc->num_queued, MIN_BUFFERS_QUEUED);
+      g_cond_wait(v4lsrc->cond_queue_state,
+                  v4lsrc->mutex_queue_state);
+    }
+    if (v4lsrc->quit) {
+      g_mutex_unlock(v4lsrc->mutex_queue_state);
+      return TRUE; /* it won't get through anyway */
+    }
+    if (!gst_v4lsrc_queue_frame(v4lsrc, v4lsrc->queue_frame)) {
+      g_mutex_unlock(v4lsrc->mutex_queue_state);
+      return FALSE;
+    }
+    v4lsrc->queue_frame = (v4lsrc->queue_frame + 1) % v4lsrc->mbuf.frames;
+  }
+
   /* syncing on the buffer grabs it */
-  if (!gst_v4lsrc_sync_next_frame(v4lsrc, num))
+  *num = v4lsrc->sync_frame;
+  if (!gst_v4lsrc_sync_frame(v4lsrc, *num)) {
+    g_mutex_unlock(v4lsrc->mutex_queue_state);
     return FALSE;
+  }
+  v4lsrc->sync_frame = (v4lsrc->sync_frame + 1) % v4lsrc->mbuf.frames;
+
+  g_mutex_unlock(v4lsrc->mutex_queue_state);
 
   return TRUE;
 }
@@ -410,8 +336,6 @@ gst_v4lsrc_grab_frame (GstV4lSrc *v4lsrc, gint *num)
 guint8 *
 gst_v4lsrc_get_buffer (GstV4lSrc *v4lsrc, gint  num)
 {
-  DEBUG("gst_v4lsrc_get_buffer(), num = %d", num);
-
   if (!GST_V4L_IS_ACTIVE(GST_V4LELEMENT(v4lsrc)) ||
       !GST_V4L_IS_OPEN(GST_V4LELEMENT(v4lsrc)))
     return NULL;
@@ -436,9 +360,23 @@ gst_v4lsrc_requeue_frame (GstV4lSrc *v4lsrc, gint  num)
   GST_V4L_CHECK_OPEN(GST_V4LELEMENT(v4lsrc));
   GST_V4L_CHECK_ACTIVE(GST_V4LELEMENT(v4lsrc));
 
-  /* and let's queue the buffer */
-  if (!gst_v4lsrc_queue_frame(v4lsrc, num))
+  /* mark frame as 'ready to requeue' */
+  g_mutex_lock(v4lsrc->mutex_queue_state);
+
+  if (v4lsrc->frame_queue_state[num] != QUEUE_STATE_SYNCED) {
+    gst_element_error(GST_ELEMENT(v4lsrc),
+                      "Invalid state %d (expected %d), can't requeue",
+                      v4lsrc->frame_queue_state[num],
+                      QUEUE_STATE_SYNCED);
     return FALSE;
+  }
+
+  v4lsrc->frame_queue_state[num] = QUEUE_STATE_READY_FOR_QUEUE;
+
+  /* let an optional wait know */
+  g_cond_broadcast(v4lsrc->cond_queue_state);
+
+  g_mutex_unlock(v4lsrc->mutex_queue_state);
 
   return TRUE;
 }
@@ -453,17 +391,27 @@ gst_v4lsrc_requeue_frame (GstV4lSrc *v4lsrc, gint  num)
 gboolean
 gst_v4lsrc_capture_stop (GstV4lSrc *v4lsrc)
 {
-  int n;
-
   DEBUG("stopping capture");
   GST_V4L_CHECK_OPEN(GST_V4LELEMENT(v4lsrc));
   GST_V4L_CHECK_ACTIVE(GST_V4LELEMENT(v4lsrc));
 
-  /* we actually need to sync on all queued buffers but not on the non-queued ones */
-  for (n=0;n<v4lsrc->mbuf.frames;n++)
-    v4lsrc->frame_queued[n] = -1;
+  g_mutex_lock(v4lsrc->mutex_queue_state);
 
-  g_thread_join(v4lsrc->thread_soft_sync);
+  /* make an optional pending wait stop */
+  v4lsrc->quit = TRUE;
+  g_cond_broadcast(v4lsrc->cond_queue_state);
+                                                                                
+  /* sync on remaining frames */
+  while (1) {
+    if (v4lsrc->frame_queue_state[v4lsrc->sync_frame] == QUEUE_STATE_QUEUED) {
+      gst_v4lsrc_sync_frame(v4lsrc, v4lsrc->sync_frame);
+      v4lsrc->sync_frame = (v4lsrc->sync_frame + 1) % v4lsrc->mbuf.frames;
+    } else {
+      break;
+    }
+  }
+
+  g_mutex_unlock(v4lsrc->mutex_queue_state);
 
   return TRUE;
 }
@@ -478,20 +426,15 @@ gst_v4lsrc_capture_stop (GstV4lSrc *v4lsrc)
 gboolean
 gst_v4lsrc_capture_deinit (GstV4lSrc *v4lsrc)
 {
-  int n;
-
   DEBUG("quitting capture subsystem");
   GST_V4L_CHECK_OPEN(GST_V4LELEMENT(v4lsrc));
   GST_V4L_CHECK_ACTIVE(GST_V4LELEMENT(v4lsrc));
 
   /* free buffer tracker */
-  g_mutex_free(v4lsrc->mutex_queued_frames);
-  for (n=0;n<v4lsrc->mbuf.frames;n++)
-    g_cond_free(v4lsrc->cond_soft_sync[n]);
-  free(v4lsrc->frame_queued);
-  free(v4lsrc->cond_soft_sync);
-  free(v4lsrc->isready_soft_sync);
-  free(v4lsrc->timestamp_soft_sync);
+  g_mutex_free(v4lsrc->mutex_queue_state);
+  g_cond_free(v4lsrc->cond_queue_state);
+  g_free(v4lsrc->frame_queue_state);
+  g_free(v4lsrc->use_num_times);
 
   /* unmap the buffer */
   munmap(GST_V4LELEMENT(v4lsrc)->buffer, v4lsrc->mbuf.size);
