@@ -34,6 +34,8 @@
 #include "gstv4l2xoverlay.h"
 #include "gstv4l2colorbalance.h"
 
+#include "gstv4l2src.h"
+
 #define DEBUG(format, args...) \
 	GST_DEBUG_OBJECT (\
 		GST_ELEMENT(v4l2element), \
@@ -84,7 +86,7 @@ gst_v4l2_fill_lists (GstV4l2Element *v4l2element)
 	if (pads && g_list_length ((GList *) pads) == 1)
 		dir = GST_PAD_DIRECTION (GST_PAD (pads->data));
 
-	if (dir == GST_PAD_SINK) {
+	if (dir != GST_PAD_SINK) {
 	/* and now, the inputs */
 		for (n=0;;n++) {
 			struct v4l2_input input;
@@ -372,6 +374,15 @@ gst_v4l2_open (GstV4l2Element *v4l2element)
 		goto error;
 	}
 
+	/* do we need to be a capture device? */
+	if (GST_IS_V4L2SRC(v4l2element) &&
+	    !(v4l2element->vcap.capabilities & V4L2_CAP_VIDEO_CAPTURE)) {
+		gst_element_error(GST_ELEMENT(v4l2element),
+				  "Not a capture device (0x%x)",
+				  v4l2element->vcap.capabilities);
+		goto error;
+	}
+
 	/* create enumerations */
 	if (!gst_v4l2_fill_lists(v4l2element))
 		goto error;
