@@ -53,8 +53,8 @@ static void		gst_xvideosink_class_init	(GstXVideoSinkClass *klass);
 static void		gst_xvideosink_init		(GstXVideoSink *xvideosink);
 static void 		gst_xvideosink_dispose 		(GObject *object);
 
-static void		gst_xvideosink_interface_init	(GstInterfaceClass *klass);
-static gboolean		gst_xvideosink_interface_supported (GstInterface *iface, GType type);
+static void		gst_xvideosink_interface_init	(GstImplementsInterfaceClass *klass);
+static gboolean		gst_xvideosink_interface_supported (GstImplementsInterface *iface, GType type);
 
 static void		gst_xvideosink_xoverlay_init	(GstXOverlayClass *klass);
 static void		gst_xvideosink_xoverlay_set_xwindow_id (GstXOverlay *overlay, XID xwindow_id);
@@ -133,7 +133,7 @@ gst_xvideosink_get_type (void)
     xvideosink_type = g_type_register_static(GST_TYPE_VIDEOSINK,
                                              "GstXVideoSink",
                                              &xvideosink_info, 0);
-    g_type_add_interface_static (xvideosink_type, GST_TYPE_INTERFACE,
+    g_type_add_interface_static (xvideosink_type, GST_TYPE_IMPLEMENTS_INTERFACE,
 	&iface_info);
     g_type_add_interface_static (xvideosink_type, GST_TYPE_NAVIGATION,
         &navigation_info);
@@ -187,13 +187,13 @@ gst_xvideosink_class_init (GstXVideoSinkClass *klass)
 }
 
 static void
-gst_xvideosink_interface_init (GstInterfaceClass *klass)
+gst_xvideosink_interface_init (GstImplementsInterfaceClass *klass)
 {
   klass->supported = gst_xvideosink_interface_supported;
 }
 
 static gboolean
-gst_xvideosink_interface_supported (GstInterface *iface, GType type)
+gst_xvideosink_interface_supported (GstImplementsInterface *iface, GType type)
 {
   g_assert (type == GST_TYPE_NAVIGATION ||
 	    type == GST_TYPE_X_OVERLAY);
@@ -895,7 +895,6 @@ GST_PLUGIN_DEFINE (
   plugin_init,
   VERSION,
   GST_LICENSE,
-  GST_COPYRIGHT,
   GST_PACKAGE,
   GST_ORIGIN
 );
